﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using TrackerGraphic;


namespace UI
{
    public class ObjectLocationDiagnostic
    {
        private const string DEFAULT_PATH = @"c:\TrackerDiag\";

        private StreamWriter _streamWriter;
        private List<Point> _truePositions;
        private List<Point> _trackersPositions;
        private List<Point> _differences;
        private List<string> _elapsedTime;
        private List<string> _iterations;
        private List<TargetArea> _areas;
        private List<double> _distances;
        private TargetArea _startLocation;

        private string _path;
        public string Path
        {
            get
            {
                if (String.IsNullOrEmpty(_path)) {
                    _path = DEFAULT_PATH;
                    if (!Directory.Exists(DEFAULT_PATH)) {
                        try {
                            Directory.CreateDirectory(DEFAULT_PATH);
                        } catch (Exception ex) {
                            _path = string.Empty;
                            System.Diagnostics.Trace.WriteLine(ex);
                        }
                    }

                }
                return _path;
            }
            set
            {
                _path = value;
            }
        }
        /// <summary>
        /// Initializes a new instance of the <see cref="TrackerDiagnostic"/> class.
        /// </summary>
        public ObjectLocationDiagnostic()
        {
            this._truePositions = new List<Point>();
            this._trackersPositions = new List<Point>();
            _differences = new List<Point>();
            this._elapsedTime = new List<string>();
            this._iterations = new List<string>();
            this._areas = new List<TargetArea>();
            _distances = new List<double>();

        }

        #region Adding methods
        /// <summary>
        /// Adds the true position.
        /// </summary>
        /// <param name="position">The position.</param>
        public void AddTruePosition(Point position)
        {
            this._truePositions.Add(position);
        }
        /// <summary>
        /// Adds the tracker position.
        /// </summary>
        /// <param name="position">The position.</param>
        public void AddTrackerPosition(Point position)
        {
            this._trackersPositions.Add(position);
        }
        /// <summary>
        /// Adds the position set.
        /// </summary>
        /// <param name="truePosition">The true position.</param>
        /// <param name="trackerPosition">The tracker position.</param>
        public void AddPositionSet(Point truePosition, Point trackerPosition)
        {
            this.AddTruePosition(truePosition);
            this.AddTrackerPosition(trackerPosition);
        }
        public void AddEllapsedTime(long milliseconds)
        {
            this._elapsedTime.Add(milliseconds.ToString());
        }
        public void AddIteration(int iteration)
        {
            this._iterations.Add(iteration.ToString());
        }
        public void AddTargetArea(TargetArea area)
        {
            this._areas.Add(area);
        }
        #endregion
        /// <summary>
        /// Clears the lists.
        /// </summary>
        public void ClearLists()
        {
            this._truePositions.Clear();
            this._trackersPositions.Clear();
            this._differences.Clear();
            this._elapsedTime.Clear();
            this._iterations.Clear();
            this._areas.Clear();
            this._distances.Clear();

        }
        /// <summary>
        /// Saves the results.
        /// </summary>
        /// <param name="path">The path.</param>
        /// <param name="fileName">Name of the file.</param>
        private void SaveResults(string path, string sequence)
        {
            this.CalcualteDifferences();
            this._streamWriter = new StreamWriter(path);
            this._streamWriter.WriteLine(sequence);
            this.WriteStartArea();
            string truePosHead = "============== True Position =========================================";
            string trackerPosHead = "============== Tracker Position ======================================";
            string differenceHead = "============== Difference (True - tracker) ===========================";
            string elapsedTime = "============================ Elapsed time [milliseconds] ================";
            string iterationsHead = "============== Iterations ============================================";
            string size = "================ Target area Size ===========================";
            string distancesHead = "================================Distances between locations =======================";
            WriteData(_truePositions, truePosHead);
            WriteData(_trackersPositions, trackerPosHead);
            WriteData(_differences, differenceHead);
            this.WriteNotSpecificData<string>(this._elapsedTime, elapsedTime);
            this.WriteNotSpecificData<string>(this._iterations, iterationsHead);
            this.WriteNotSpecificData<double>(this._distances, distancesHead);

            this.WriteAreas(this._areas, size);

            this._streamWriter.Close();
            this.ClearLists();
        }
        /// <summary>
        /// Ulozi obsah vsech seznamu do souboru
        /// </summary>
        /// <param name="name">The name.</param>
        /// <param name="startArea">The start area.</param>
        public void Save(string name, TargetArea startArea)
        {
            this._startLocation = startArea;
            string path = this.PrepareFile(this.Path);
            if (!String.IsNullOrEmpty(path)) {
                this.SaveResults(path, name);
            }
        }

        #region Writing methods

        /// <summary>
        /// Writes the data.
        /// </summary>
        /// <param name="list">The list.</param>
        /// <param name="head">The head.</param>
        private void WriteData(List<Point> list, string head)
        {
            this._streamWriter.WriteLine(head);
            this._streamWriter.WriteLine("\n ================ X - coordinates ================");

            foreach (Point item in list) {
                this._streamWriter.WriteLine(item.X.ToString());
            }
            this._streamWriter.WriteLine("\n =============== Y - coordinates =================");
            foreach (Point item in list) {
                this._streamWriter.WriteLine(item.Y.ToString());
            }
        }
        /// <summary>
        /// Writes the areas.
        /// </summary>
        /// <param name="list">The list.</param>
        /// <param name="head">The head.</param>
        private void WriteAreas(List<TargetArea> list, string head)
        {
            this._streamWriter.WriteLine(head.ToUpper());
            this._streamWriter.WriteLine("\n ================ Width ================");
            foreach (TargetArea item in list) {
                this._streamWriter.WriteLine(item.ObjectArea.Width.ToString());
            }

            this._streamWriter.WriteLine("\n =============== Height =================");
            foreach (TargetArea item in list) {
                this._streamWriter.WriteLine(item.ObjectArea.Height.ToString());
            }

        }
        /// <summary>
        /// Writes the not specific data.
        /// </summary>
        /// <param name="list">The list.</param>
        /// <param name="head">The head.</param>
        private void WriteNotSpecificData<T>(List<T> list, string head)
        {
            this._streamWriter.WriteLine("\n ================ " + head.ToUpper() + "  ================");
            foreach (var item in list) {
                this._streamWriter.WriteLine(item.ToString());
            }
        }
        /// <summary>
        /// Writes the start area.
        /// </summary>
        private void WriteStartArea()
        {
            if (_startLocation == null) {
                return;
            }
            this._streamWriter.WriteLine("Target Area initializing parameters ");
            this._streamWriter.WriteLine(String.Format("Start location X-{0}, Y-{1} ... End location X-{2}, Y-{3}, Width {4}, Height {5}  ", this._startLocation.ObjectArea.UpperLeftCorner.X,
                this._startLocation.ObjectArea.UpperLeftCorner.Y, this._startLocation.ObjectArea.BottomRightCorner.X, this._startLocation.ObjectArea.BottomRightCorner.Y,
                this._startLocation.ObjectArea.Height, this._startLocation.ObjectArea.Width));
        }
        #endregion

        /// <summary>
        /// Calcualtes the differences.
        /// </summary>
        private void CalcualteDifferences()
        {
            Point difference;
            double distance = 0.0;
            if (this._trackersPositions.Count != 0 && this._truePositions.Count != 0) {
                if (this._trackersPositions.Count != this._truePositions.Count)
                    throw new IndexOutOfRangeException("nesouhlasi delky seznamu");
                for (int i = 0; i < this._truePositions.Count; i++) {
                    difference = PointDifference(this._truePositions[i], this._trackersPositions[i]);
                    distance = CalcualteDistance(difference.X, difference.Y);
                    this._differences.Add(difference);
                    this._distances.Add(distance);
                }
            }
        }
        /// <summary>
        /// Vypocita rozdil mezi dvemi body
        /// </summary>
        /// <param name="p1">The p1.</param>
        /// <param name="p2">The p2.</param>
        /// <returns></returns>
        private Point PointDifference(Point p1, Point p2)
        {
            return new Point(p1.X - p2.X, p1.Y - p2.Y);
        }
        private static double CalcualteDistance(int width, int height)
        {
            double distance = Math.Sqrt(Math.Pow(Math.Abs(width), 2) + Math.Pow(Math.Abs(height), 2));
            return distance;
        }

        private string PrepareFile(string path)
        {
            string filePath = string.Empty;
            try {
                DirectoryInfo dir = new DirectoryInfo(path);
                if (!dir.Exists) {
                    dir.Create();
                }
                int fileOrder = dir.GetFiles().Length;
                filePath = path + @"\" + fileOrder.ToString() + ".txt";
                FileInfo info = new FileInfo(filePath);
                var a = info.Create();
                a.Close();
                a.Dispose();
                a = null;
            } catch (Exception ex) {
                System.Diagnostics.Debug.WriteLine(ex.Message);
            }
            return filePath;
        }

    }
}
