﻿using System;
using System.Drawing;

namespace RGBHistogramGenerator
{
    /// <summary>
    /// Bázová třída pro vytváření histogramů
    /// </summary>
    public abstract class BaseHistogram
    {
        #region Fields
        /// <summary>
        /// počet R binů
        /// </summary>
        private int _numberOfBinsRed;
        /// <summary>
        /// počet G binů
        /// </summary>
        private int _numberOfBinsGreen;
        /// <summary>
        /// počet B binů
        /// </summary>
        private int _numberOfBinsBlue;
        #endregion

        #region Properties
        /// <summary>
        /// Get/Set Počet binů pro R kanál 
        /// </summary>
        public virtual int NumberOfBinsRed
        {
            get
            {
                return this._numberOfBinsRed;
            }
            protected set
            {
                this._numberOfBinsRed = value;
            }
        }
        /// <summary>
        /// Get/Set Počet binů pro G kanál 
        /// </summary>
        public virtual int NumberOfBinsGreen
        {
            get
            {
                return this._numberOfBinsGreen;
            }
            protected set
            {
                this._numberOfBinsGreen = value;
            }
        }
        /// <summary>
        /// Get/Set Počet binů pro B kanál 
        /// </summary>
        public virtual int NumberOfBinsBlue
        {
            get
            {
                return this._numberOfBinsBlue;
            }
            protected set
            {
                this._numberOfBinsBlue = value;
            }
        }
        #endregion

        #region Methods
        /// <summary>
        /// Zkontroluje zda je cislo binu do ktereho se zapisuje validni, pripadne opravi
        /// </summary>
        /// <param name="bin"></param>
        /// <param name="max"></param>
        /// <returns></returns>
        protected static int ValidateBins(int bin, int max)
        {
            max -= 1;
            if (bin - max > 0) {
                return max;
            }
            return bin;
            //return Math.Min(bin, max - 1);
        }

        #region Abstract
        /// <summary>
        /// Get sestaveny histogram
        /// </summary>
        /// <value>normalizovaný histogram</value>
        public abstract Array Histogram
        {
            get;
            set;

        }
        /// <summary>
        /// Výpočet histogramu.
        /// </summary>
        /// <param name="bmp">zdrojová bitmapa.</param>
        /// <returns>histogram</returns>
        public abstract BaseHistogram CalculateHistogram(ref Bitmap bmp);

        /// <summary>
        /// Ziská frekvenci vyskytu predaneho barevneho odstinu v histogramu
        /// </summary>
        /// <param name="red">intenzita cervene</param>
        /// <param name="green">intenzita zelene.</param>
        /// <param name="blue">intenzita modre.</param>
        /// <returns></returns>
        public abstract Single GetColorFrequency(byte red, byte green, byte blue);
        /// <summary>
        /// Zapise vyskyt predaneho barevneho odstinu na prislusnou pozici v histogramu.
        /// </summary>
        /// <param name="r">intenzita cervene</param>
        /// <param name="g">intenzita zelene</param>
        /// <param name="b">intenzita modre</param>
        /// <param name="value">hodnota ktera se ma v histogramu pricist na prislusnou pozici.</param>
        public abstract void AddColorValue(byte r, byte g, byte b, Single value);
        /// <summary>
        /// Normalizuje  histogram.
        /// </summary>
        /// <param name="colours">pocet barev histogramu.</param>
        public abstract void NormaliseHistogram(Single colours);
        /// <summary>
        /// Ziska nejmensi zapsanou nenulovou hodnotu histogramu
        /// </summary>
        /// <returns>nejmensi hodnota vb histogramu</returns>
        public abstract Single GetSmallestNonzeroEntry();
        /// <summary>
        /// Ziska nejvetsi zapsanou nenulovou hodnotu histogramu
        /// </summary>
        /// <returns>nejvetsi hodnota v histogramu</returns>
        public abstract Single GetLargestEntry();
        #endregion//abstrtact

        #region Constructors

        /// <summary>
        /// Konstruktor
        /// </summary>
        /// <param name="numberOfBinsRed">Počet binů pro R slozku</param>
        /// <param name="numberOfBinsGreen">Počet binů pro G slozku</param>
        /// <param name="numberOfBinsBlue">Počet binů pro B slozku</param>
        public BaseHistogram(int numberOfBinsRed, int numberOfBinsGreen, int numberOfBinsBlue)
        {
            this._numberOfBinsRed = numberOfBinsRed;
            this._numberOfBinsGreen = numberOfBinsGreen;
            this._numberOfBinsBlue = numberOfBinsBlue;
        }
        public BaseHistogram()
        {
        }
        #endregion//construcotrs
        #endregion // methods


    }
}
