﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using TrackerGraphic;

namespace RGBHistogramGenerator
{

    /// <summary>
    /// Třída slouží k výpočtu pozice těžiště daného obrázku na základě hodnot pixelu z histogramu
    /// cíle a histogramu kandidáta.
    /// </summary>
    public static class DistributionCalculator
    {
        private static Single zeroMoment;
        private static Single firstMomentX;
        private static Single firstMomentY;
        private static Single firstMomentXY;
        private static Single secondMomentX;
        private static Single secondMomentY;
        private static DistributionData disData;
        private static byte r, g, b;
        /// <summary>
        /// Resetuje staticke pole
        /// </summary>
        private static void ResetFields()
        {
            zeroMoment = firstMomentX = firstMomentY =
                secondMomentX = secondMomentY = firstMomentXY = 0;
        }
        /// <summary>
        /// Získa souradnice bodu vypocitaneho teziste
        /// </summary>
        /// <param name="sourceImage">Obrazek z ktereho se ma vypocitat teziste</param>
        /// <param name="targetHistogram">histogram cile</param>
        /// <param name="candidateHistogram">histogram kandidata</param>
        /// <returns>Souřadnice těžiště</returns>
        public static Point GetCentroidPosition(Bitmap sourceImage, BaseHistogram targetHistogram,
            BaseHistogram candidateHistogram)
        {
            ResetFields();
            InputValidation(sourceImage, targetHistogram, candidateHistogram);
            return GetCentroid(sourceImage, targetHistogram, candidateHistogram);
        }
        /// <summary>
        /// Validace vstupu na null
        /// </summary>
        /// <param name="sourceImage">Obrazek z ktereho se ma vypocitat teziste</param>
        /// <param name="targetHistogram">histogram cile</param>
        /// <param name="candidateHistogram">histogram kandidata</param>
        private static void InputValidation(Bitmap sourceImage, BaseHistogram targetHistogram,
            BaseHistogram candidateHistogram)
        {
            if (null == sourceImage || null == targetHistogram || null == candidateHistogram)
                throw new NullReferenceException();
        }
        /// <summary>
        /// Metoda projde jednotlive pixely obrazku, porovna do jake miry si histogramy pro dany pixel podobaji 
        /// a vypocita jednotlivou vahu vzorku, ktery je pak pouzit k vypoctu pozice teziste pro dany objekt na
        /// obrazku.
        /// </summary>
        private static unsafe Point GetCentroid(Bitmap sourceImage, BaseHistogram targetHistogram,
            BaseHistogram candidateHistogram)
        {
            Single weight = 0;
            BitmapData data = sourceImage.LockBits(
                new Rectangle(0, 0, sourceImage.Width, sourceImage.Height),
                ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            int* pos;
            byte r, g, b;
            for (int y = 0; y < sourceImage.Height; y++) {
                pos = (int*)((int)data.Scan0 + (y * data.Stride));
                for (int x = 0; x < data.Width; x++) {
                    BackProjection.IntArgbToByteValues(pos, out r, out g, out b);
                    weight = CalculateWeight(targetHistogram.GetColorFrequency(r, g, b),
                        candidateHistogram.GetColorFrequency(r, g, b));
                    zeroMoment += weight;
                    firstMomentX += weight * x;
                    firstMomentY += weight * y;
                    pos++;
                }
            }
            sourceImage.UnlockBits(data);
            return GetCentroidPosition(zeroMoment, firstMomentX, firstMomentY/*, area*/);
        }



        /// <summary>
        /// Ziska udaje o statickych momentech a velikostech objektu, z obrazu zpetne projekce
        /// </summary>
        /// <param name="sourceImage">Obraz zpetne projekce.</param>
        /// <returns>data barevne distribuce</returns>
        public static unsafe DistributionData GetDistributionInformations(Bitmap sourceImage)
        {
            ResetFields();
            disData = new DistributionData();
            BitmapData data = sourceImage.LockBits(
                new Rectangle(0, 0, sourceImage.Width, sourceImage.Height),
                ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            byte r;
            int* pos;
            for (int y = 0; y < sourceImage.Height; y++) {
                pos = (int*)((int)data.Scan0 + (y * data.Stride));
                for (int x = 0; x < data.Width; x++) {
                    r = (byte)((*pos >> 16) & 0xFF);
                    disData.AddValueToRecalculation(r, x, y);
                    pos++;
                }
            }
            sourceImage.UnlockBits(data);
            return disData.GetDistributionData();
        }

        /// <summary>
        /// Vypocet vahoveho vzorku z hodnot histogramu cile a histogramu kandidata na dane stejne pozici
        /// </summary>
        /// <param name="targetHistogramValue">hodnota z histogramu cile</param>
        /// <param name="candidateHistogramValue">hodnota z histogramu kandidata</param>
        /// <returns>váhový vzorek</returns>
        private static Single CalculateWeight(Single targetHistogramValue, Single candidateHistogramValue)
        {
            if (candidateHistogramValue > 0)
                return (Single)Math.Sqrt(targetHistogramValue / candidateHistogramValue);
            else
                return 0;
        }
        /// <summary>
        /// Vypocet souradnic posunu teziste X a Y
        /// </summary>
        /// <param name="weight">vaha</param>
        /// <param name="firstMomentX">wX</param>
        /// <param name="firstMomentY">wY</param>
        public static Point GetCentroidPosition(Single weight, Single firstMomentX, Single firstMomentY)
        {
            if (weight > 0) {
                Point centroidPosition = new Point();
                centroidPosition.X = Convert.ToInt32(firstMomentX / weight);
                centroidPosition.Y = Convert.ToInt32(firstMomentY / weight);
                return centroidPosition;
            }
            return new Point();
        }
        /// <summary>
        /// Prepocita souradnice teziste k predane oblasti cile. Vypocita velikosti posunuti oblasti
        /// </summary>
        /// <param name="centroidPosition">souradnice teziste </param>
        /// <param name="area">oblast ke ktere se ma teziste prepocitat</param>
        /// <returns>prepocitane teyiste</returns>
        public static Point RecalculateCentroidToArea(Point centroidPosition, TargetArea area)
        {
            int x = area.ObjectArea.AreaRectangle.Width / 2;
            int y = area.ObjectArea.AreaRectangle.Height / 2;
            centroidPosition.X -= x;
            centroidPosition.Y -= y;
            return centroidPosition;
        }



    }
}
