﻿using System;
using System.Drawing;

namespace RGBHistogramGenerator
{
    public class DistributionData
    {
        #region Constatns
        public const double COEFICITN_SIZE = 1.9;
        private const Single DEGREES_180 = 180;
        private const Single DEGREES_360 = 360;
        #endregion

        #region Fields
        private Point _centroidPosition;
        private Size _distributionSize;
        private Single _zeroMoment;
        private Single _firstMomentX;
        private Single _firstMomentY;
        private Single _firstMomentXY;
        private Single _secondMomentX;
        private Single _secondMomentY;
        private double _angle;
        private Size _size;
        #endregion //endfields

        #region Properties
        /// <summary>
        /// Get Pozice těžiště
        /// </summary>
        public Point CentroidPosition
        {
            get
            {
                return this._centroidPosition;
            }
        }
        /// <summary>
        /// Get Rozměry sledovaneho objektu
        /// </summary>
        public Size DistributionSize
        {
            get
            {
                return this._distributionSize;
            }
        }
        /// <summary>
        /// Get Nulty moment
        /// </summary>
        public Single ZeroMoment
        {
            get
            {
                return this._zeroMoment;
            }
        }
        /// <summary>
        /// Get První moment pro x - ovou osu
        /// </summary>
        public Single FirstMomentX
        {
            get
            {
                return this._firstMomentX;
            }
        }
        /// <summary>
        /// Get První moment pro y-ovou osu
        /// </summary>
        public Single FirstMomentY
        {
            get
            {
                return this._firstMomentY;
            }
        }
        /// <summary>
        /// Get První moment pro obe osy
        /// </summary>
        public Single FirstMomentXY
        {
            get
            {
                return this._firstMomentXY;
            }
        }
        /// <summary>
        /// Get Druhý moment pro x-ovou osu
        /// </summary>
        public Single SecondMomentX
        {
            get
            {
                return this._secondMomentX;
            }
        }
        /// <summary>
        /// Get Druhý moment pro y-ovou osu
        /// </summary>
        public Single SecondMomentY
        {
            get
            {
                return this._secondMomentY;
            }
        }
        /// <summary>
        /// Get Uhel natočení objektu
        /// </summary>
        public double Angle
        {
            get
            {
                return this._angle;
            }
        }
        public Size Size
        {
            get
            {
                return this._size;
            }
        }

        /// <summary>
        /// Get Uhel natoceni distribuce v radianech
        /// </summary>
        /// <value>The angle in radians.</value>
        public double AngleInRadians
        {
            get
            {
                return ((Math.PI / DEGREES_180) * this._angle);
            }
        }
        /// <summary>
        /// Get uhel natoceni distribuce ve stupnich
        /// </summary>
        /// <value>The angle in degrees.</value>
        public double AngleInDegrees
        {
            get
            {
                return (Single)((DEGREES_180 / Math.PI)) * this._angle;
            }
        }
        #endregion //end properties


        #region Constructors
        public DistributionData()
        {
        }
        /// <summary>
        /// Initializes a new instance of the <see cref="CAMShiftData"/> struct.
        /// </summary>
        /// <param name="zeroMoment">The zero moment.</param>
        /// <param name="firstMomentX">The first moment X.</param>
        /// <param name="firstMomentY">The first moment Y.</param>
        /// <param name="firstMomentXY">The first moment XY.</param>
        /// <param name="secondMomentX">The second moment X.</param>
        /// <param name="secondMomentY">The second moment Y.</param>
        public DistributionData(Single zeroMoment, Single firstMomentX, Single firstMomentY,
            Single firstMomentXY, Single secondMomentX, Single secondMomentY)
        {
            this._zeroMoment = ValidateInput(zeroMoment);
            this._firstMomentX = ValidateInput(firstMomentX);
            this._firstMomentY = ValidateInput(firstMomentY);
            this._firstMomentXY = ValidateInput(firstMomentXY);
            this._secondMomentX = ValidateInput(secondMomentX);
            this._secondMomentY = ValidateInput(secondMomentY);
            this._centroidPosition = DistributionCalculator.GetCentroidPosition(this._zeroMoment,
                this._firstMomentX, this._firstMomentY);

            this.CalculatAngle();
            this.CalculateSize();
            this._size = this.GetWindowSize();
            // Debug.WriteLine((int)this.AngleInDegrees);

        }

        /// <summary>
        /// Zahrne predanou hodnotu do vypoctu statickych momentu 
        /// </summary>
        /// <param name="r">barevna hodnota z obrazu zpetne projekce.</param>
        /// <param name="x"> poloha x</param>
        /// <param name="y">poloha y</param>
        public void AddValueToRecalculation(byte r, int x, int y)
        {
            this._zeroMoment += r;
            this._firstMomentX += r * x;
            this._firstMomentY += r * y;
            this._firstMomentXY += r * x * y;
            this._secondMomentX += r * x * x;
            this._secondMomentY += r * y * y;
        }
        /// <summary>
        /// Zvaliduje a vradi strukturu, ktera obsahuje udaje o momentech, velikostech a uhlu natoceni 
        /// barevne reprezentace objektu
        /// </summary>
        /// <returns>Data barevne distribuce</returns>
        public DistributionData GetDistributionData()
        {
            this._zeroMoment = ValidateInput(_zeroMoment);
            this._firstMomentX = ValidateInput(_firstMomentX);
            this._firstMomentY = ValidateInput(_firstMomentY);
            this._firstMomentXY = ValidateInput(_firstMomentXY);
            this._secondMomentX = ValidateInput(_secondMomentX);
            this._secondMomentY = ValidateInput(_secondMomentY);
            this._centroidPosition = DistributionCalculator.GetCentroidPosition(this._zeroMoment,
                this._firstMomentX, this._firstMomentY);
            //this.CalculatAngle();
            this.CalculateSize();
            this._size = this.GetWindowSize();
            return this;
        }
        #endregion //endconstructors

        #region Methods
        /// <summary>
        /// Validates the input.
        /// </summary>
        /// <param name="value">The value.</param>
        /// <returns></returns>
        private Single ValidateInput(Single value)
        {
            if (value <= 0)
                value = 1;
            return value;
        }
        /// <summary>
        /// Prevede uhel ve stupnich na radiany
        /// </summary>
        /// <param name="angle">uhel ve stupnich</param>
        /// <returns>uhel v radianech</returns>
        private Single ToRadians(Single angle)
        {
            return (Single)((Math.PI / DEGREES_180) * angle);

        }
        /// <summary>
        /// Prevede uhel v radianech na stupne
        /// </summary>
        /// <param name="angle">uhel v radianech</param>
        /// <returns>uhel ve stupnich</returns>
        private Single ToDegrees(Single angle)
        {
            return (Single)((DEGREES_180 / Math.PI) * angle);
        }
        /// <summary>
        /// Vzpocitani uhlu natoceni distribuce
        /// </summary>
        private void CalculatAngle()
        {
            int x = this._centroidPosition.X;
            int y = this._centroidPosition.Y;
            Single numerator = 2 * ((this._firstMomentXY / this._zeroMoment) - (x * y));
            Single denominatorL = ((this._secondMomentX / this._zeroMoment) - (x * x));
            Single denominatorR = ((this._secondMomentY / this._zeroMoment) - (y * y));
            this._angle = (Single)(Math.Atan(numerator / (denominatorL - denominatorR)) / 2);
#warning odebrat tady
            Single val = this.ToRadians((Single)this._angle);
            Single val2 = this.ToDegrees((Single)this._angle);
        }

        /// <summary>
        /// Vypocitani rozmeru distribuce
        /// </summary>
        private void CalculateSize()
        {
            int x = this._centroidPosition.X;
            int y = this._centroidPosition.Y;

            double a = (this._secondMomentX / this._zeroMoment) - (Math.Pow(x, 2));
            double b = 2 * ((this._firstMomentXY / this._zeroMoment) - (x * y));
            double c = ((this._secondMomentY / this._zeroMoment) - Math.Pow(y, 2));
            double temp = Math.Sqrt(Math.Pow(b, 2) + (Math.Pow(a, 2) - (2 * a * c) + Math.Pow(c, 2)));
            double height = Math.Sqrt(((a + c) + temp) / 2);
            double width = Math.Sqrt(((a + c) - temp) / 2);
            this._angle = 0.5 * Math.Atan(b / (a - c));
            double s = Math.Sqrt(this._zeroMoment / 256.0) * COEFICITN_SIZE;
            double fWidth = Math.Sqrt(height / width) * s;
            double fHeight = Math.Sqrt(width / height) * s;
            //double angle;
            //angle = this._angle;
            //double oWidth = (Math.Cos(angle) * fWidth) + (fHeight * Math.Sin(angle));
            //double oHeight = (Math.Sin(angle) * fWidth) + (fHeight * Math.Cos(angle));
            this._distributionSize = new Size((int)fHeight, (int)fWidth);
        }
        /// <summary>
        /// Vrati jednoduchou velikost sledovaci oblasti - sirka okna je nastavena pro oba rozmery
        /// </summary>
        /// <returns></returns>
        private Size GetWindowSize()
        {
            int width = (int)Math.Sqrt(this._zeroMoment / (256.0)) * 2;
            return new Size(width, width);
        }

        public override string ToString()
        {
            return String.Format("Centroid: {0} , Size {1}", this.CentroidPosition, this.DistributionSize);
        }

        #endregion //end methods
    }
}
