﻿using System;
using System.Diagnostics;
using System.Drawing;
using RGBHistogramGenerator.Enumerations;
using TrackerGraphic;

namespace RGBHistogramGenerator
{
    /// <summary>
    /// Třída HistogramGenerator umožňuje generovat jednotlivé histogramy z Bitmapy, pro správnou funkci je potřeba zadat
    /// počet binů pro jednotlivé barevné RGB kanaly a zdrojový typ obrázku. Typ zdroje indikuje, zda se jedna o hledaný cíl,
    /// nebo o kandidáta.
    /// </summary>
    public unsafe class HistogramGenerator
    {
        #region Fields
        /// <summary>
        /// Histogram cíle
        /// </summary>
        private BaseHistogram _targetHistogram;
        /// <summary>
        /// Typ generovaných histogramů
        /// </summary>
        private HistogramType _generatorType;
        /// <summary>
        /// Histogram kandidata
        /// </summary>
        private BaseHistogram _candidateHistogram;
        private BaseHistogram _wholeImageHistogram;
        #endregion

        #region Property
        /// <summary>
        /// Get Histogram kandidáta
        /// </summary>
        public BaseHistogram CandidateHistogram
        {
            get
            {
                return this._candidateHistogram;
            }
        }
        /// <summary>
        /// Get cílový histogram
        /// </summary>
        /// <value>cíl.</value>
        public BaseHistogram Target
        {
            get
            {
                return this._targetHistogram;
            }
        }
        /// <summary>
        /// Get Typ generátoru - zda se jedna o 1D nebo 3D histogram
        /// </summary>
        /// <value>typ generatoru.</value>
        public HistogramType GeneratorType
        {
            get
            {
                return this._generatorType;
            }
        }

        /// <summary>
        /// GetHistogram celeho snimku
        /// </summary>
        public BaseHistogram WholeImage
        {
            get
            {
                return this._wholeImageHistogram;
            }
        }
        #endregion

        #region Constructors
        /// <summary>
        /// Konstruktor
        /// </summary>
        /// <param name="type">Typ generátoru - zda se bude jednat o 1D nebo 3D histogramy</param>
        public HistogramGenerator(HistogramType type)
        {
            this._generatorType = type;
        }
        #endregion // end constructors

        #region Methods

        /// <summary>
        /// Generuje histogramy podle typu zdroje
        /// </summary>
        /// <param name="NumberOfRedBins"></param>
        /// <param name="NumberOfGreenBins"></param>
        /// <param name="NumberOfBluedBins"></param>
        /// <param name="type"></param>
        public BaseHistogram GenerateHistogram(int NumberOfRedBins, int NumberOfGreenBins, int NumberOfBlueBins,
            SourceType type, ref Bitmap bmp, TargetArea area)
        {
            if ((NumberOfRedBins < 1) || (NumberOfGreenBins < 1) || (NumberOfBlueBins < 1))
                throw new FormatException("Rozmery histogramu musí nabývat hodnot větších než 1.");
            if (type == SourceType.Target) {
                GenerateTarget(NumberOfRedBins, NumberOfGreenBins, NumberOfBlueBins, ref bmp, area);
                return this._targetHistogram;
            } else if (type == SourceType.Candidate) {
                this.GenerateCandidates(NumberOfRedBins, NumberOfGreenBins, NumberOfBlueBins, ref bmp);
                return this._candidateHistogram;
            } else {
                this.GenerateWholeImage(NumberOfRedBins, NumberOfGreenBins, NumberOfBlueBins, ref bmp);
                return this._wholeImageHistogram;
            }
        }
        /// <summary>
        /// Generuje histogram na zaklade dopredu znamych poctu Binu, ktere byly zadany pri prvotnim
        /// generování cíloveho histogramu.
        /// </summary>
        /// <param name="type">typ o jaky histogram se jedna, zda cíle, nebo kandidáta</param>
        /// <param name="bmp">bitmapa z ktere bude histogram generovan</param>
        /// <returns></returns>
        public BaseHistogram GenerateHistogram(SourceType type, ref Bitmap bmp, TargetArea area)
        {
            Debug.Assert(null != this._targetHistogram && CheckBinSettings(), "Neni vytvorena instance ciloveho histogramu, nebo neni nastaven pocet Binu");
            return this.GenerateHistogram(this._targetHistogram.NumberOfBinsRed,
                this._targetHistogram.NumberOfBinsGreen, this._targetHistogram.NumberOfBinsBlue,
                type, ref bmp, area);
        }
        public BaseHistogram GenerateHistogram(SourceType type, ref Bitmap bmp)
        {
            Debug.Assert(null != this._targetHistogram && CheckBinSettings(), "Neni vytvorena instance ciloveho histogramu, nebo neni nastaven pocet Binu");
            return this.GenerateHistogram(this._targetHistogram.NumberOfBinsRed,
                this._targetHistogram.NumberOfBinsGreen, this._targetHistogram.NumberOfBinsBlue,
                type, ref bmp, null);
        }
        /// <summary>
        /// Zkontroluje, zda je nastaven pocet Binu
        /// </summary>
        /// <returns></returns>
        private bool CheckBinSettings()
        {
            return this._targetHistogram.NumberOfBinsRed != 0 &&
                this._targetHistogram.NumberOfBinsGreen != 0 &&
                this._targetHistogram.NumberOfBinsBlue != 0;
        }
        /// <summary>
        /// Generuje histogramy kandidátů ze zadaná bitmapy
        /// </summary>
        /// <param name="NumberOfRedBins">The number of red bins.</param>
        /// <param name="NumberOfGreenBins">The number of green bins.</param>
        /// <param name="NumberOfBluedBins">The number of blued bins.</param>
        /// <param name="bmp">odkaz na bitmapu</param>
        private void GenerateCandidates(int NumberOfRedBins, int NumberOfGreenBins, int NumberOfBluedBins, ref Bitmap bmp)
        {
            //switch (this.m_generatorType)
            //{
            //    case HistogramType.Unweighted:
            //        this.m_candidateHistogram = new HistogramUnweighted(NumberOfRedBins, NumberOfGreenBins,
            //        NumberOfBluedBins).CalculateHistogram(ref bmp);
            //        break;
            //    case HistogramType.KernelBased:
            //        this.m_candidateHistogram = new HistogramKernelBased(NumberOfRedBins, NumberOfGreenBins,
            //        NumberOfBluedBins).CalculateHistogram(ref bmp);
            //        break;
            //    default: throw new NullReferenceException("Neznamy typ histogramu");
            //}
            this._candidateHistogram = new HistogramUnweighted(NumberOfRedBins, NumberOfGreenBins,
                NumberOfBluedBins).CalculateHistogram(ref bmp);
        }
        /// <summary>
        /// Generuje cíle
        /// </summary>
        /// <param name="NumberOfRedBins">Počet binů na červenou škálu</param>
        /// <param name="NumberOfGreenBins">Počet binů na zelenou škálu</param>
        /// <param name="NumberOfBluedBins">Počet binů na modrou škálu</param>
        private void GenerateTarget(int NumberOfRedBins, int NumberOfGreenBins, int NumberOfBluedBins, ref Bitmap bmp, TargetArea area)
        {
            switch (this._generatorType) {
                case HistogramType.Unweighted:
                    this._targetHistogram = new HistogramUnweighted(NumberOfRedBins, NumberOfGreenBins,
                    NumberOfBluedBins).CalculateHistogram(ref bmp);
                    break;
                case HistogramType.KernelBased:
                    this._targetHistogram = new HistogramKernelBased(NumberOfRedBins, NumberOfGreenBins,
                    NumberOfBluedBins).CalculateHistogram(ref bmp);
                    break;
                case HistogramType.Weighted:
                    if (area != null) {
                        this._targetHistogram = new HistogramWeighted(NumberOfRedBins, NumberOfGreenBins,
                        NumberOfBluedBins, area, KernelProfilesLibrary.EpanchikovKernel, KernelProfilesLibrary.EpanchikovKernel).CalculateHistogram(ref bmp);

                        //this.m_targetHistogram = new HistogramWeighted(NumberOfRedBins, NumberOfGreenBins,
                        //NumberOfBluedBins, area, KernelProfilesLibrary.DefaultForegroundProfile, KernelProfilesLibrary.DefaultBackGroundProfile).CalculateHistogram(ref bmp);

                        break;
                    }
                    break;
                default:
                    throw new NullReferenceException("Neznamy typ histogramu");
            }
        }
        /// <summary>
        /// Generuje histogram pro cely obrazek
        /// </summary>
        /// <param name="NumberOfRedBins"></param>
        /// <param name="NumberOfGreenBins"></param>
        /// <param name="NumberOfBluedBins"></param>
        private void GenerateWholeImage(int NumberOfRedBins, int NumberOfGreenBins, int NumberOfBluedBins, ref Bitmap bmp)
        {
            this._wholeImageHistogram = new HistogramUnweighted(NumberOfRedBins, NumberOfGreenBins,
                NumberOfBluedBins).CalculateHistogram(ref bmp);
        }
        /// <summary>
        /// Vypocet Bhattacharyya Coefficient pro 3D histogram
        /// </summary>
        /// <param name="target">target</param>
        /// <param name="candidate">candidate</param>
        /// <returns>Bhattacharyya Coefficient</returns>
        private static double BhattacharyyaCoefficient3D(BaseHistogram target, BaseHistogram candidate)
        {
            double result = 0;
            for (int z = 0; z < target.NumberOfBinsBlue; z++) {
                for (int y = 0; y < target.NumberOfBinsGreen; y++) {
                    for (int x = 0; x < target.NumberOfBinsRed; x++) {
                        result += Math.Sqrt(((Single[, ,])target.Histogram)[x, y, z] *
                            ((Single[, ,])candidate.Histogram)[x, y, z]);
                    }
                }
            }
            return result;
        }
        /// <summary>
        /// Metoda pro vypocet Bhattacharyya Coefficient, implicitne se rozhodne o jaky histogram se jedna 
        /// jestli 1d nebo 3d
        /// </summary>
        /// <param name="target"></param>
        /// <param name="candidate"></param>
        /// <returns></returns>
        public static double BhattacharyyaCoefficient(BaseHistogram target, BaseHistogram candidate)
        {
            if ((target.Histogram as Single[, ,]) != null)
                return BhattacharyyaCoefficient3D(target, candidate);
            else
                return 0;
        }
        /// <summary>
        /// Vypočítá Bhattacharyyho koeficient pro cíl a kandidáta, cíl je implicitně daný
        /// </summary>
        /// <param name="candidate">kandidát</param>
        /// <returns>Bhattacharyyho koeficient</returns>
        public double MatchHistograms(BaseHistogram candidate)
        {
            return BhattacharyyaCoefficient(this._targetHistogram, candidate);
        }
        /// <summary>
        /// Porovnava histogramy implicitne, bere vzdy histogram kandidata z prvni pozice seznamu
        /// </summary>
        /// <returns>Bhattacharyyho koeficient</returns>
        public double MatchHistograms()
        {
            return this.MatchHistograms(this._candidateHistogram);
        }
        /// <summary>
        /// Vypočítá a vratí vzdálenost dvou histogramu - vzdalenost mezi cilem a kandidatem
        /// </summary>
        /// <returns>vzdálenost objektu</returns>
        public double GetDistributionDistance()
        {
            return Math.Sqrt(1 - this.MatchHistograms());
        }
        #endregion

    }
}
