﻿using System;
using System.Drawing;
using System.Drawing.Imaging;

namespace RGBHistogramGenerator
{
    /// <summary>
    /// Histogram class
    /// </summary>
    public class HistogramUnweighted : BaseHistogram
    {
        /// <summary>
        /// Celkovy pocet barevnych hodnot na složku
        /// </summary>
        private const Single MAXIMAL_COLOR_VALUE = 256;
        /// <summary>
        /// Pole histogramu
        /// </summary>
        private Single[, ,] _histogramArray;
        private int _R;
        private int _G;
        private int _B;
        private int _coloursPerRed;
        private int _coloursPerGreen;
        private int _coloursPerBlue;
        private Single _smallestEntry;
        private Single _largestEntry;
        /// <summary>
        /// Get Pole 3DHistogramu
        /// </summary>
        public Single[, ,] HistogramArray
        {
            get
            {
                return this._histogramArray;
            }
            set
            {
                this._histogramArray = value;
            }
        }
        /// <summary>
        /// Konstruktor - inicializuje pole histogramu
        /// </summary>
        /// <param name="numberOfRedBins">pocet binu pro červenou barvu</param>
        /// <param name="numberOfGreenBins">pocet binu pro zelenou barvu</param>
        /// <param name="numberOfBlueBins">pocet binu pro modrou barvu</param>
        public HistogramUnweighted(int numberOfRedBins, int numberOfGreenBins, int numberOfBlueBins)
            : base(numberOfRedBins, numberOfGreenBins, numberOfBlueBins)
        {
            this._histogramArray = new Single[base.NumberOfBinsRed,
                base.NumberOfBinsGreen, base.NumberOfBinsBlue];
            this._coloursPerRed = this.GetColoursPerBin(this.NumberOfBinsRed);
            this._coloursPerGreen = this.GetColoursPerBin(this.NumberOfBinsGreen);
            this._coloursPerBlue = this.GetColoursPerBin(this.NumberOfBinsBlue);
            this._smallestEntry = Single.MaxValue;
            this._largestEntry = Single.MinValue;


        }
        /// <summary>
        /// Zapise hodnotu predane barvy na vypocitanou pozici histogramu
        /// </summary>
        /// <param name="pixel">barva pixelu</param>
        /// <param name="value">inkrementacni hodnota</param>
        public void AddColorValue(Color pixel, Single value)
        {
            this.AddColorValue(pixel.R, pixel.G, pixel.B, value);
        }

        /// <summary>
        /// Zapise barvu na prislusnou polohu histogramu
        /// </summary>
        /// <param name="red">intenzita cervene</param>
        /// <param name="green">intenzita zelene</param>
        /// <param name="blue">intenzita modre.</param>
        /// <param name="value">hodnota ktera ma byt zapsana</param>
        public override void AddColorValue(byte red, byte green, byte blue, Single value)
        {
            _R = (red / this._coloursPerRed);
            _G = (green / this._coloursPerGreen);
            _B = (blue / this._coloursPerBlue);
            this._histogramArray[ValidateBins(_R, NumberOfBinsRed), ValidateBins(_G, NumberOfBinsGreen), ValidateBins(_B, NumberOfBinsBlue)] += value;
        }

        //public void AddColorValue(double red, double green, double blue, Single value)
        //{

        //    _R = (int)(red / this._coloursPerRed);
        //    _G = (int)(green / this._coloursPerGreen);
        //    _B = (int)(blue / this._coloursPerBlue);
        //    this._histogramArray[ValidateBins(_R, NumberOfBinsRed), ValidateBins(_G, NumberOfBinsGreen), ValidateBins(_B, NumberOfBinsBlue)] += value;
        //}

        /// <summary>
        /// Získá počet barev, ktere zastupuje jeden bin
        /// </summary>
        /// <param name="binnumbers">velikost binu</param>
        /// <returns>pocet barev na bin</returns>
        public int GetColoursPerBin(int binnumbers)
        {
            return (int)(MAXIMAL_COLOR_VALUE / binnumbers);
        }
        /// <summary>
        /// Get Normalizovany histogram
        /// </summary>
        public override Array Histogram
        {
            get
            {
                return this._histogramArray;
            }
            set
            {
                this._histogramArray = value as Single[, ,];
            }
        }
        /// <summary>
        /// Vypočítá histogram z předané bitmapy
        /// </summary>
        /// <param name="bmp">bitmapa z ktere ma byt vypocitan histogram</param>
        /// <returns>vygenerovaný histogram</returns>
        public unsafe override BaseHistogram CalculateHistogram(ref System.Drawing.Bitmap bmp)
        {
            int colours = 0;
            BitmapData data = bmp.LockBits(
                new Rectangle(0, 0, bmp.Width, bmp.Height),
                ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            int* pos;
            byte r, g, b;
            for (int y = 0; y < data.Height; y++) {
                // vypocita pozice, kde zacinaji data daneho radku 
                pos = (int*)(data.Scan0.ToInt32() + (y * data.Stride));
                for (int x = 0; x < data.Width; x++) {
                    BackProjection.IntArgbToByteValues(pos, out r, out g, out b);
                    this.AddColorValue(r, g, b, 1);

                    pos++;
                    colours++;
                }
            }
            bmp.UnlockBits(data);
            this.NormaliseHistogram(colours);
            return this;
        }
        /// <summary>
        /// Získá frekvenci výskytu předane barvy na obrazku
        /// </summary>
        /// <param name="c">barva pixelu</param>
        /// <returns>počet výskytu barvy</returns>
        /// 
        [Obsolete("pouzivat metodu s parametrem byte - je rychlejsi")]
        public Single GetColorFrequency(Color c)
        {
            return this.GetColorFrequency(c.R, c.G, c.B);
        }
        public override float GetColorFrequency(byte red, byte green, byte blue)
        {
            _R = (red / this._coloursPerRed);
            _G = (green / this._coloursPerGreen);
            _B = (blue / this._coloursPerBlue);
            return this.HistogramArray[ValidateBins(_R, NumberOfBinsRed)
                , ValidateBins(_G, NumberOfBinsGreen),
                ValidateBins(_B, NumberOfBinsBlue)];
        }
        /// <summary>
        /// Normalizuje Histogram podle predaneho poctu barev
        /// </summary>
        /// <param name="colours">pocet barev v histogramu</param>
        public override void NormaliseHistogram(Single colours)
        {
            for (int blue = 0; blue < base.NumberOfBinsBlue; blue++) {
                for (int green = 0; green < base.NumberOfBinsGreen; green++) {
                    for (int red = 0; red < base.NumberOfBinsRed; red++) {
                        if (this._histogramArray[red, green, blue] != 0) {
                            this._histogramArray[red, green, blue] /= colours;
                        }
                    }
                }
            }
        }
        /// <summary>
        /// Ziska nejmensi hodnotu histogramu
        /// </summary>
        /// <returns>nejmensi hodnota histogramu</returns>
        public override Single GetSmallestNonzeroEntry()
        {
            _smallestEntry = Single.MaxValue;
            Single temp;
            for (int blue = 0; blue < base.NumberOfBinsBlue; blue++) {
                for (int green = 0; green < base.NumberOfBinsGreen; green++) {
                    for (int red = 0; red < base.NumberOfBinsRed; red++) {
                        if (this._histogramArray[red, green, blue] != 0) {
                            temp = this._histogramArray[red, green, blue];
                            _smallestEntry = (temp < _smallestEntry) ? temp : _smallestEntry;
                        }
                    }
                }
            }
            return _smallestEntry;

        }
        public override Single GetLargestEntry()
        {
            _largestEntry = Single.MinValue;
            Single temp;
            for (int blue = 0; blue < base.NumberOfBinsBlue; blue++) {
                for (int green = 0; green < base.NumberOfBinsGreen; green++) {
                    for (int red = 0; red < base.NumberOfBinsRed; red++) {
                        if (this._histogramArray[red, green, blue] != 0) {
                            temp = this._histogramArray[red, green, blue];
                            _largestEntry = (temp > _largestEntry) ? temp : _largestEntry;
                        }
                    }
                }
            }
            return _largestEntry;
        }
    }
}
