﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using TrackerGraphic;

namespace RGBHistogramGenerator
{
    public delegate Single KernelProfileCalculation(Single distance);
    public class HistogramWeighted : HistogramUnweighted
    {

        /// <summary>
        /// Histogram cile
        /// </summary>
        private static BaseHistogram _targetHistogram;
        /// <summary>
        /// Histogram pozadi
        /// </summary>
        private static BaseHistogram _backgroundHistogram;
        /// <summary>
        /// Oblast
        /// </summary>
        private TargetArea m_targetArea;
        private KernelProfileCalculation _kernelForeground;
        private KernelProfileCalculation _kernelBackground;
        private static Point _center;
        private static Single _distance;
        private static Single _pixels;
        private static Single _Xn;
        private static Single _Yn;
        private static BitmapData _data;
        private static Single _value;
        private static int _bgSum;
        private static int _fgSum;
        private static Single _radius;

        public HistogramWeighted(int red, int green, int blue, TargetArea targetArea,
            KernelProfileCalculation foregroundProfileDelegate, KernelProfileCalculation backgroundProfileDelegate)
            : this(red, green, blue, targetArea)
        {
            this._kernelForeground = foregroundProfileDelegate;
            this._kernelBackground = backgroundProfileDelegate;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="HistogramWeighted"/> class.
        /// </summary>
        /// <param name="red">The red bins.</param>
        /// <param name="green">The green bins.</param>
        /// <param name="blue">The blue bins.</param>
        /// <param name="targetArea">The target area.</param>
        public HistogramWeighted(int red, int green, int blue, TargetArea targetArea)
            : base(red, green, blue)
        {
            this.m_targetArea = targetArea;
        }
        /// <summary>
        /// Initializes a new instance of the <see cref="HistogramWeighted"/> class.
        /// </summary>
        /// <param name="red">The red.</param>
        /// <param name="green">The green.</param>
        /// <param name="blue">The blue.</param>
        private HistogramWeighted(int red, int green, int blue)
            : base(red, green, blue)
        {
        }
        /// <summary>
        /// Initicializuje pomocne histogramy
        /// </summary>
        private void Initialize()
        {
            _targetHistogram = new HistogramWeighted(base.NumberOfBinsRed, base.NumberOfBinsGreen, base.NumberOfBinsBlue);
            _backgroundHistogram = new HistogramWeighted(base.NumberOfBinsRed, base.NumberOfBinsGreen, base.NumberOfBinsBlue);
            _bgSum = 0;
            _fgSum = 0;
            _distance = 0;
            _value = 0;
            _Xn = 0;
            _Yn = 0;
            _radius = 0;
            _pixels = 0;
        }
        /// <summary>
        /// Vypocita histogram
        /// </summary>
        /// <param name="bmp">obrazek</param>
        /// <returns>Vypocitany histogram</returns>
        public override unsafe BaseHistogram CalculateHistogram(ref System.Drawing.Bitmap bmp)
        {
            this.Initialize();
            _center = new Point(bmp.Width / 2, bmp.Height / 2);
            _data = bmp.LockBits(
                new Rectangle(0, 0, bmp.Width, bmp.Height),
                ImageLockMode.ReadOnly, PixelFormat.Format32bppArgb);
            int* pos;
            byte r, g, b;
            for (int y = 0; y < _data.Height; y++) {
                pos = (int*)(_data.Scan0.ToInt32() + (y * _data.Stride));
                for (int x = 0; x < _data.Width; x++) {
                    BackProjection.IntArgbToByteValues(pos, out r, out g, out b);
                    _Xn = (Single)2 * (x - _center.X) / bmp.Width;
                    _Yn = (Single)2 * (y - _center.Y) / bmp.Height;
                    _distance = (Single)Math.Sqrt(_Xn * _Xn + _Yn * _Yn);
                    _radius = _distance * _distance;
                    if (m_targetArea.ObjectArea.Contains(x + this.m_targetArea.SearchingArea.UpperLeftCorner.X,
                        (y + this.m_targetArea.SearchingArea.UpperLeftCorner.Y))) {
                        if (this._kernelForeground != null) {
                            _value = this._kernelForeground(_distance);
                        } else {
                            _value = KernelProfilesLibrary.EpanchikovKernel(_distance);
                        }
                        _targetHistogram.AddColorValue(r, g, b, _value);

                        _fgSum++;
                    } else {
                        if (this._kernelBackground != null) {
                            _value = this._kernelBackground(_distance);
                        } else {
                            _value = KernelProfilesLibrary.EpanchikovKernel(_distance);
                        }
                        _backgroundHistogram.AddColorValue(r, g, b, _value);

                        _bgSum++;
                    }
                    _pixels++;
                    pos++;
                }
            }
            bmp.UnlockBits(_data);
            //_targetHistogram.NormaliseHistogram(_bgSum);
            //_backgroundHistogram.NormaliseHistogram(_fgSum);
            CalculateWieghtedHistogram();
            _targetHistogram = null;
            _backgroundHistogram = null;
            return this;
        }
        /// <summary>
        /// Vypocet vyvazeneho histogramu
        /// </summary>
        private void CalculateWieghtedHistogram()
        {
            WeightBackground();
            WeightForeground();
            this.Histogram = _targetHistogram.Histogram;

        }
        /// <summary>
        /// Vyvazeni popredi
        /// </summary>
        private void WeightForeground()
        {
            Single sum = 0;
            Single[, ,] targetArray = _targetHistogram.Histogram as Single[, ,];
            Single[, ,] bgArray = _backgroundHistogram.Histogram as Single[, ,];
            for (int blue = 0; blue < base.NumberOfBinsBlue; blue++) {
                for (int green = 0; green < base.NumberOfBinsGreen; green++) {
                    for (int red = 0; red < base.NumberOfBinsRed; red++) {
                        targetArray[red, green, blue] *= bgArray[red, green, blue];
                        sum += targetArray[red, green, blue];
                    }
                }
            }
            (_targetHistogram as HistogramWeighted).HistogramArray = targetArray;
            //bylo predtim
            //_targetHistogram.NormalisedHistogram = targetArray;
            if (sum > 0)
                _targetHistogram.NormaliseHistogram(sum);
        }
        /// <summary>
        /// Vyvazeni pozadi
        /// </summary>
        private void WeightBackground()
        {
            Single minNonZero = _backgroundHistogram.GetSmallestNonzeroEntry();
            Single value = 0;
            Single sum = 0;
            Single[, ,] bgArray = _backgroundHistogram.Histogram as Single[, ,];
            for (int blue = 0; blue < base.NumberOfBinsBlue; blue++) {
                for (int green = 0; green < base.NumberOfBinsGreen; green++) {
                    for (int red = 0; red < base.NumberOfBinsRed; red++) {
                        if (bgArray[red, green, blue] > 0) {
                            value = Math.Min((minNonZero / bgArray[red, green, blue]), 1);
                        } else
                            value = 1;
                        bgArray[red, green, blue] = value;
                        sum += value;
                    }
                }
            }
            _backgroundHistogram.Histogram = bgArray;
        }
    }
}
