﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Threading;

namespace UI.Buffer
{
    /// <summary>
    /// SINGLETON - Trida buffer slouzi jako zasobnik pro soubory ziskavane z kameroveho zarizeni pri realtime zpracovani.
    /// Buffer je implementovan pomoci fronty, kdy jsou do ni ukladany nove snimky sekvence, ktere cekaji na zpracovani. 
    /// Nabizi verejne metody AddItem a GetItem, pro manipulaci se s nimky. V pripade ze snimky jsou vcas odebirany a buffer
    /// nema prekrocenou kapacitu, potom se ziskavane snimky zapisuji do pameti. Pokud snimky rychleji "prichazeji" nez 
    /// je jejich frekvence zpracovani, pak dochazi k ukladani na pevny disk. Odtud jsou docasne soubory odstraneny, jakmile jsou zpracovany
    /// Vychozi umisteni docasnych souboru je C:\TrackerBufferTemporary  , tato slozka je po ukonceni aplikace odstranena, pripadne je vycistena 
    /// pri startu aplikace
    /// </summary>
    public class BufferManager
    {
        private event EventHandler CleanCalled;
        #region Constants
        private const string DEFAULT_PATH = @"C:\\TrackerBufferTemporary";
        private const int DEFAULT_CAPACITY = 30;
        #endregion

        #region Properties

        #region ImageQueue
        /// <summary>
        /// Get / Set Fronta obrazu
        /// </summary>
        /// <value>
        /// The image queue.
        /// </value>
        private Queue<Bitmap> ImageQueue
        {
            get;
            set;
        }
        #endregion

        #region Counter
        /// <summary>
        /// Get / set citadlo zpracovanych obrazu
        /// </summary>
        /// <value>
        /// The image counter.
        /// </value>
        private int ImageCounter
        {
            get;
            set;
        }
        #endregion

        #region Destination Path
        private string _path;
        /// <summary>
        /// Get / Set Cilove umisteni sekvence
        /// </summary>
        /// <value>
        /// The path.
        /// </value>
        public string Path
        {
            get
            {
                if (String.IsNullOrEmpty(_path)) {
                    this._path = DEFAULT_PATH;
                }
                return this._path;
            }
            set
            {
                this._path = value;
            }
        }
        #endregion

        #region Instance - Singleton
        private static BufferManager _instance;
        /// <summary>
        /// Get singleton instance.
        /// </summary>
        public static BufferManager Instance
        {
            get
            {
                if (_instance == null) {
                    _instance = new BufferManager(DEFAULT_CAPACITY);
                }

                return _instance;
            }

        }
        #endregion

        #region Reserve
        /// <summary>
        /// Get pocet snimku, ktery je Bufferem rezervovan
        /// </summary>
        private int Reserve
        {
            get
            {
                return this.Capacity / 2;
            }
        }
        #endregion

        #region Capacity
        /// <summary>
        /// Get / Set kapacita
        /// </summary>
        /// <value>
        /// The capacity.
        /// </value>
        private int Capacity
        {
            get;
            set;
        }
        #endregion

        #region BuffersActivated
        /// <summary>
        /// Get / Set  priznak , zda je buffer aktivni
        /// </summary>
        /// <value>
        ///   <c>true</c> v pripade ze je aktivni; jinak, <c>false</c>.
        /// </value>
        private bool BuffersActivated
        {
            get;
            set;
        }
        #endregion

        #region IsReadyToLoad
        private bool _isReadyToLoad;
        /// <summary>
        /// Get / Set zda je buffer pripraven na doplneni fronty snimku
        /// </summary>
        /// <value>
        /// 	true jestlize je buffer pripraven nacist snimky
        /// </value>
        private bool IsReadyToLoad
        {
            get
            {
                return this._isReadyToLoad;
            }
            set
            {
                if (value) {
                    this.StartLoading();
                }
                this._isReadyToLoad = value;
            }
        }
        #endregion

        #region IsEmpty

        public bool IsEmpty
        {
            get
            {
                return this.ImageQueue.Count == 0;
            }
        }
        #endregion

        #region IsLocked
        /// <summary>
        /// Pokud je True, prichozi snimky budou ignorovany
        /// </summary>
        private bool IsLocked
        {
            get;
            set;
        }
        #endregion
        #endregion

        #region Fields
        private FileInfo[] _toDelete;
        private DirectoryInfo _directoryInfo;
        private FileInfo[] _fileInfos;
        #endregion

        /// <summary>
        /// Soukromy konstruktor v dusledku jedinacka
        /// </summary>
        /// <param name="capacity">The capacity.</param>
        private BufferManager(int capacity)
        {
            _directoryInfo = new DirectoryInfo(this.Path);
            if (_directoryInfo.Exists) {
                try {
                    var files = _directoryInfo.GetFiles();
                    for (int i = 0; i < files.Length; i++) {
                        files[i].Delete();
                    }
                    _directoryInfo.Delete();
                } catch (Exception ex) {
                    System.Diagnostics.Debug.WriteLine(ex.Message);
                }
            }
            _directoryInfo.Create();
            this.Capacity = capacity;
            this.ImageQueue = new Queue<Bitmap>(Capacity + this.Reserve);
            this.CleanCalled += new EventHandler(BufferManager_LoadingPrepared);
        }

        #region Events handlers
        /// <summary>
        /// Obsluzna metoda udalosti, kdy je oznameno, ze nacteni snimku do pameti probehlo.
        /// Reaguje se na to vymazanim docasnych souboru z disku 
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void BufferManager_LoadingPrepared(object sender, EventArgs e)
        {
            this.DeleteUsedFiles();
        }
        #endregion

        /// <summary>
        /// Prida snimek do fronty. V pripade ze je fronta plna, zapise soubor na docasne uloziste na disku.
        /// </summary>
        /// <param name="item">The item.</param>
        public void Add(Bitmap item)
        {
            if (IsLocked) {
                return;
            }
            if (!Directory.Exists(this.Path)) {
                try {
                    _directoryInfo.Create();
                } catch (Exception ex) {
                    System.Diagnostics.Debug.WriteLine(ex.Message);
                }
            }
            if (this.ImageQueue.Count < this.Capacity + this.Reserve && !this.BuffersActivated) {
                this.ImageQueue.Enqueue(item);
            } else {
                this.BuffersActivated = true;
                item.Save(this.Path + "\\" + this.ImageCounter.ToString(), ImageFormat.Jpeg);
                item.Dispose();
                this.ImageCounter++;
            }
        }


        /// <summary>
        /// Ziska snimek z bufferu a odebere jej z fronty
        /// </summary>
        /// <returns></returns>
        public Bitmap GetFrame()
        {
            if (this.ImageQueue.Count > 0) {
                if (this.ImageQueue.Count <= this.Capacity && this.BuffersActivated) {
                    this.IsReadyToLoad = true;
                }
                return this.ImageQueue.Dequeue();
            }
            return null;
        }
        /// <summary>
        /// Zapocne nacitani obrazku z uloziste do pameti.
        /// </summary>
        private void StartLoading()
        {
            int capacity = (this.Capacity + this.Reserve) - this.ImageQueue.Count;
            //ThreadPool.QueueUserWorkItem(w => {
                //Thread.Sleep(250);
                _fileInfos = _directoryInfo.GetFiles();
                _fileInfos = Array.FindAll<FileInfo>(_fileInfos, f => f.Exists/* && (Convert.ToInt32(f.Name) > lastIndex)*/);
                Array.Sort<FileInfo>(_fileInfos, (f1, f2) => (Convert.ToInt32(f1.Name)).CompareTo(Convert.ToInt32(f2.Name)));
                Bitmap img = null;
                this._toDelete = new FileInfo[capacity];
                for (int i = 0; i < capacity; i++) {
                    if (!_fileInfos[i].Exists)
                        continue;
                    using (img = Bitmap.FromFile(_fileInfos[i].FullName) as Bitmap) {
                        this.ImageQueue.Enqueue(img.GetThumbnailImage(img.Width, img.Height, null, IntPtr.Zero) as Bitmap);
                    }
                    //lastIndex = Convert.ToInt32(_fileInfos[i].Name);
                    img = null;
                    //img = Bitmap.FromFile(_fileInfos[i].FullName) as Bitmap;
                    //this.ImageQueue.Enqueue(img.GetThumbnailImage(img.Width, img.Height, null, IntPtr.Zero) as Bitmap);
                    //img.Dispose();
                    if (i <= _toDelete.Length - 1) {
                        _toDelete[i] = _fileInfos[i];
                    } else {
                        break;
                    }
                }
                GC.Collect();
                GC.WaitForPendingFinalizers();
                GC.Collect();

                this.OnCleanCalled();
            //});
        }

        /// <summary>
        /// Odpaleni udalosti CleanCalled. Oznamuje, ze se ma provest mazani docasnych souboru
        /// </summary>
        private void OnCleanCalled()
        {
            if (this.CleanCalled != null) {
                this.CleanCalled(null, new EventArgs());
            }
        }
        List<FileInfo> _cantDelete;
        /// <summary>
        /// Odstrani jiz zpracovane soubory z docasneho uloziste. Process si vyazada samostatne jadro
        /// </summary>
        private void DeleteUsedFiles()
        {
            if (_cantDelete == null) {
                _cantDelete = new List<FileInfo>();
            }
            ThreadPool.QueueUserWorkItem(w => {
                for (int i = 0; i < this._toDelete.Length; i++) {
                    FileInfo info = _toDelete[i];
                    try {
                        if (info != null) {

                            info.Delete();
                        }
                    } catch (Exception ex) {
                        System.Diagnostics.Debug.WriteLine(ex.Message);
                        _cantDelete.Add(info);
                    }
                }
            });
        }

        /// <summary>
        /// Vyresetuje buffer a vsechny ptislusne fieldy, odstrani docasne soubory
        /// </summary>
        public void Reset()
        {
            this.IsLocked = true;
            ThreadPool.QueueUserWorkItem(w => {
                if (this._directoryInfo != null && Directory.Exists(this.Path)) {
                    foreach (FileInfo item in this._directoryInfo.GetFiles()) {
                        item.Delete();
                    }
                    this._directoryInfo.Delete(true);
                }
                this.ImageQueue.Clear();
                this.BuffersActivated = false;
                this.ImageCounter = 0;
                this.IsLocked = false;
            });
        }



    }
}
