﻿using System;
using System.Drawing;
using System.Windows.Forms;
using AForge.Video;
using AForge.Video.DirectShow;

namespace UI
{
    public sealed class FrameCapturer : IDisposable
    {

        private static FrameCapturer _frameCapturer;
        private VideoCaptureDevice _vcDevice;
        private FilterInfoCollection _filterCollection;
        private Timer _camDetector;
        private int _devicesCount;

        public delegate void FrameCaptureEventHandler(object sender, Bitmap e);
        public event FrameCaptureEventHandler FrameCaptured;
        public event EventHandler DeviceFound;
        /// <summary>
        /// Gets the video capture devices.
        /// </summary>
        /// <value>The video capture devices.</value>
        public VideoCaptureDevice VideoCaptureDevices
        {
            get
            {
                return this._vcDevice;
            }
        }
        /// <summary>
        /// Gets the filter info collection.
        /// </summary>
        /// <value>The filter info collection.</value>
        public FilterInfoCollection FilterInfoCollection
        {
            get
            {
                return this._filterCollection;
            }
        }
        /// <summary>
        /// Gets a value indicating whether this instance is running.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is running; otherwise, <c>false</c>.
        /// </value>
        public bool IsRunning
        {
            get
            {
                return this._vcDevice.IsRunning;
            }
        }
        /// <summary>
        /// Gets a value indicating whether is device set.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is device set; otherwise, <c>false</c>.
        /// </value>
        public bool IsDeviceSet
        {
            get
            {
                return !String.IsNullOrEmpty(this._vcDevice.Source);
            }
        }
        private Size _frameSize;

        public Size FrameSize
        {
            get
            {
                return _frameSize;
            }
            set
            {
                _frameSize = value;
                Reset();
            }
        }
        private static bool _isDefaultResoulutionSelected;

        public static bool IsDefaultResoulutionSelected
        {
            get
            {
                return _isDefaultResoulutionSelected;
            }
            set
            {
                _isDefaultResoulutionSelected = value;
            }
        }



        /// <summary>
        /// Initializes a new instance of the <see cref="FrameCapturer"/> class.
        /// </summary>
        public FrameCapturer()
        {
            this._vcDevice = new VideoCaptureDevice();
            this._filterCollection = new FilterInfoCollection(FilterCategory.VideoInputDevice);
            this._vcDevice.NewFrame += new NewFrameEventHandler(m_vcDevice_NewFrame);
            this._camDetector = new Timer();
            this._camDetector.Interval = 1000;
            this._camDetector.Tick += new EventHandler(m_camDetector_Tick);
            this.Reset();
        }

        private void Reset()
        {
            //if (FrameCapturer.IsDefaultResoulutionSelected) {
            //    this.m_vcDevice.DesiredFrameSize = new Size(320, 240);
            //} else {
            //    this.m_vcDevice.DesiredFrameSize = new Size(640, 480);
            //}
            this._devicesCount = 0;
            this._camDetector.Start();
        }
        /// <summary>
        /// Handles the Tick event of the m_camDetector control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void m_camDetector_Tick(object sender, EventArgs e)
        {
            this._filterCollection = new FilterInfoCollection(FilterCategory.VideoInputDevice);
            if (this._filterCollection.Count != this._devicesCount) {
                this._devicesCount = this._filterCollection.Count;
                this.OnDeviceFound(new DevicesEventArgs(this._filterCollection));
            } else {
                if (this._filterCollection.Count == 0 && this.IsRunning)
                    this.Stop();
            }
        }
        /// <summary>
        /// Handles the NewFrame event of the m_vcDevice control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="eventArgs">The <see cref="AForge.Video.NewFrameEventArgs"/> instance containing the event data.</param>
        void m_vcDevice_NewFrame(object sender, NewFrameEventArgs eventArgs)
        {
            this.OnFrameCaptured(eventArgs);
        }
        /// <summary>
        /// Gets the instance.
        /// </summary>
        /// <returns></returns>
        public static FrameCapturer GetInstance()
        {
            if (_frameCapturer == null)
                _frameCapturer = new FrameCapturer();
            _frameCapturer.Reset();
            return _frameCapturer;
        }
        /// <summary>
        /// Sets the device.
        /// </summary>
        /// <param name="index">The index.</param>
        public void SetDevice(int index)
        {
            if (index <= _filterCollection.Count - 1) {
                this._vcDevice.Source = _filterCollection[index].MonikerString;
            }
        }
        /// <summary>
        /// Raises the <see cref="E:FrameCaptured"/> event.
        /// </summary>
        /// <param name="e">The <see cref="AForge.Video.NewFrameEventArgs"/> instance containing the event data.</param>
        private void OnFrameCaptured(NewFrameEventArgs e)
        {
            Bitmap bmp = e.Frame.Clone() as Bitmap;
            if (IsDefaultResoulutionSelected) {
                if (e.Frame.Width != 320 || e.Frame.Height != 240) {
                    bmp = e.Frame.GetThumbnailImage(320, 240, null, IntPtr.Zero) as Bitmap;
                }
            } else {
                if (e.Frame.Width != 640 || e.Frame.Height != 480) {
                    bmp = e.Frame.GetThumbnailImage(640, 480, null, IntPtr.Zero) as Bitmap;
                }
            }

            if (this.FrameCaptured != null) {
                this.FrameCaptured(this, bmp);
            }
        }
        /// <summary>
        /// Raises the <see cref="E:DeviceFound"/> event.
        /// </summary>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnDeviceFound(EventArgs e)
        {
            if (DeviceFound != null)
                DeviceFound(this, e);
        }
        /// <summary>
        /// Starts this instance.
        /// </summary>
        public void Start()
        {
            if (!this._vcDevice.IsRunning && this._vcDevice.Source != null)
                this._vcDevice.Start();
        }
        /// <summary>
        /// Stops this instance.
        /// </summary>
        public void Stop()
        {
            if (this._vcDevice.IsRunning) {
                this._camDetector.Stop();
                this._camDetector.Tick -= new EventHandler(m_camDetector_Tick);
                this._vcDevice.NewFrame -= new NewFrameEventHandler(m_vcDevice_NewFrame);
                System.Threading.ThreadPool.QueueUserWorkItem(s => {
                    try {
                        this._vcDevice.Stop();
                    } catch (Exception ex) {
                        System.Diagnostics.Debug.WriteLine(ex.Message);
                    }

                });
            }
        }

        public void ChangeResolution(bool defaultMode)
        {
            FrameCapturer.IsDefaultResoulutionSelected = defaultMode;

        }

        public void ChangeFrameRate(int rate)
        {
            this._vcDevice.DesiredFrameRate = rate;
        }
        #region IDisposable Members

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            //if (this._vcDevice.IsRunning) {
            //    //this._vcDevice.Stop();
            //}
        }

        #endregion
    }
}
