﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Text;
using System.Threading;
using System.Windows.Forms;
using Diagnostic;
using RGBHistogramGenerator.Enumerations;
using Tracker;
using TrackerGraphic;
using UI;
using UI.Buffer;
using UI.Properties;


namespace RGBHistogramGenerator
{
    public partial class ObjectTrackingForm : Form
    {

        #region Constants
        private const bool SAVE_IMAGES = false;
        private const int PEN_WIDTH = 2;
        /// <summary>
        /// Vychozi nastaveni casvace
        /// </summary>
        private const int TIMER_INTERVAL = 25;
        /// <summary>
        /// Pocet snimku k nacteni
        /// </summary>
        private const int NUMBER_OF_FRAMES = int.MaxValue;
        /// <summary>
        /// Pocet binu histogramu pro cervenou barvu
        /// </summary>
        private const int R = 6;
        /// <summary>
        /// Pocet binu histogramu pro zelenou barvu
        /// </summary>
        private const int G = 6;
        /// <summary>
        /// Pocet binu histogramu pro modrou barvu
        /// </summary>
        private const int B = 6;
        /// <summary>
        /// Popisek tlacitka Start
        /// </summary>
        private const string START = "Start";
        /// <summary>
        /// Popisek tlacitka stop
        /// </summary>
        private const string STOP = "Stop";
        /// <summary>
        /// Koncovka souboru obrazku
        /// </summary>
        private const string FILENAME_EXTENSION = ".jpg";
        /// <summary>
        /// Vychozi hodnota pro zmensovani/ zvetsovani adaptivniho okna
        /// </summary>
        private const string DEFAULT_RATIO = "0";
        /// <summary>
        /// Prostor mezi pictureBoxem a  GroupBoxy s ovladanim
        /// </summary>
        private const int WINDOW_MIDDLE_PADDING = 100;
        private const int DEFAULT_PADDING = 20;
        /// <summary>
        /// Prostor mezi pictureBoxem a Tab controlem
        /// </summary>
        private readonly Color TARGET_COLOR = Color.GreenYellow;
        private readonly Color SEARCHING_AREA_COLOR = Color.BlueViolet;
        private readonly Color TARGET_COLOR_BACKPROJECTION = Color.Red;
        private readonly Size DEFAULT_SIZE = new Size(800, 640);
        private readonly Size DEFAULT_PICTUREBOX_SIZE = new Size(50, 50);
        #endregion

        #region Fields
        /// <summary>
        /// Citac odeslanych obrazku
        /// </summary>
        private int _imageCounter = 0;
        /// <summary>
        /// Seznam nactenych obrazku k analyze
        /// </summary>
        private List<Bitmap> _images;
        /// <summary>
        /// Cesta k souborum
        /// </summary>
        private string _path;
        /// <summary>
        /// Casovac k odesilani obrazku
        /// </summary>
        private System.Windows.Forms.Timer _timerImageSender;
        /// <summary>
        /// Meanshift tracker
        /// </summary>
        private MeanShift _meanShift;
        /// <summary>
        /// Pero pro vykreslovani pozice cile
        /// </summary>
        private Pen _penObject;
        /// <summary>
        /// Pero pro vykreslovani prohledavaci oblasti
        /// </summary>
        private Pen _penSearch;
        /// <summary>
        /// Pero pro vykreslovani pozice cile na BackProjection
        /// </summary>
        private Pen _penObjectBP;
        /// <summary>
        /// Field obsluhujici vyber cile
        /// </summary>
        private TargetSelection _selection;
        /// <summary>
        /// Hodnota pro zmensovani/ zvetsovani adaptivniho okna
        /// </summary>
        private int _scaleRatio;
        /// <summary>
        /// Priznak reprezentujici zda sekvence proehla do konce
        /// </summary>
        private bool _endSequence;
        /// <summary>
        /// Tooltip ke controlum
        /// </summary>
        private ToolTip _toolTip;
        private int _redBins = R;
        private int _greenBins = G;
        private int _blueBins = B;
        //private List<Point> m_centroidList;
        private FrameCapturer _frameCapturer;
        private bool _isRealTrackingRunning;
        private RecordForm _recodForm;
        private string _sequenceName;


        #endregion

        #region Properties
        #region IsEndSequence
        /// <summary>
        /// Get Příznak, zda již sekvence proběhla až do konce
        /// </summary>
        public bool IsEndSequence
        {
            get
            {
                return this._endSequence;
            }
        }
        #endregion

        #region IsPlaying
        public bool IsPlaying
        {
            get;
            private set;
        }
        #endregion

        #region IsFirstAidSelected

        private bool IsFirstAidSelected
        {
            get
            {
                return AdaptiveTargetArea.IsFirstAidEnabled;
            }
            set
            {
                AdaptiveTargetArea.IsFirstAidEnabled = value;
            }
        }
        #endregion

        #region SenderInterval
        public int SenderInterval
        {
            get
            {
                if (this._timerImageSender != null)
                    return this._timerImageSender.Interval;
                else
                    return 0;
            }
            private set
            {
                if (value > 0)
                    this._timerImageSender.Interval = value;
                else
                    this._timerImageSender.Interval = TIMER_INTERVAL;
            }
        }
        #endregion

        #region IsRealTimeSelected
        private bool _isRealTimeSelected;
        /// <summary>
        /// Gets a value indicating whether this instance is real time selected.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is real time selected; otherwise, <c>false</c>.
        /// </value>
        public bool IsRealTimeSelected
        {
            get
            {
                //return this.tcSource.SelectedTab == this.tpRealTimeOT;
                return this._isRealTimeSelected;
            }
            private set
            {
                this._isRealTimeSelected = value;
            }

        }
        #endregion

        #endregion

        ObjectLocationDiagnostic _diagnostic;
        /// <summary>
        /// Initializes a new instance of the <see cref="ObjectTrackingForm"/> class.
        /// </summary>
        public ObjectTrackingForm()
        {
            _diagnostic = new ObjectLocationDiagnostic();
            InitializeComponent();

            BackProjectionForm.Instance.OnWindowClosed += (e) =>
                this.menuInnerBP.Checked = !e;
            TrackingDetailsForm.Instance.OnWindowClosed += (e) =>
                this.menuTrackingDetails.Checked = !e;
            this.menuComboBoxTrackers.SelectedIndex = 1;
            this.menuInner320x240.Checked = true;
            this.cbHistogramType.DataSource = new HistogramType[] { HistogramType.Unweighted, HistogramType.KernelBased, HistogramType.Weighted };
            this._timerImageSender = new System.Windows.Forms.Timer();
            this._penObject = new Pen(TARGET_COLOR, PEN_WIDTH);
            this._penSearch = new Pen(SEARCHING_AREA_COLOR, PEN_WIDTH);
            this._penObjectBP = new Pen(TARGET_COLOR_BACKPROJECTION, PEN_WIDTH);
            this.Size = DEFAULT_SIZE;
            this.LoadSettings();
            FrameCapturer.IsDefaultResoulutionSelected = Settings.Default.IsDefaultResolution;
            this.InitializeHistogramTextBoxes();
            this._toolTip = new ToolTip();
            this.btnStart.Tag = false;
            if (String.IsNullOrEmpty(this.txeTimerInterval.Text))
                this.txeTimerInterval.Text = TIMER_INTERVAL.ToString();

            this.lblScaleAdaptation.Visible = this.lblPercent.Visible = this.txeScaleRatio.Visible = this.chbSaveImages.Visible = false;
            this.btnMark.Visible = this.txtTopLeft.Visible = false;
            this.chbDiagnostic.Visible = true;
            #region Obsluzne metody udalosti
            //zamezeni uprav textu comboboxu
            this.cbHistogramType.KeyPress += (sender, e) =>
                e.Handled = true;
            //na odpaleni udalosti OnTick zasila snimek k analyze            
            this._timerImageSender.Tick += (sender, e) =>
                this.SendImageToAnalyze();
            this.txeTimerInterval.TextChanged += new EventHandler(textBox1_TextChanged);
            this.txeScaleRatio.TextChanged += new EventHandler(txeScaleRatio_TextChanged);
            this.pictureBoxTracker.MouseMove += new MouseEventHandler(pictureBox1_MouseMove);
            this.pictureBoxRealTracking.MouseMove += new MouseEventHandler(pictureBoxRealTracking_MouseMove);
            this.pictureBoxTracker.SizeChanged += (sender, e) =>
                this.ResizeForm();
            this.pictureBoxRealTracking.SizeChanged += new EventHandler(pictureBoxRealTracking_SizeChanged);
            this.FormClosing += (sender, e) => {
                this.SaveSettings();
                this.pictureBoxRealTracking.Dispose();
            };
            this.menuItemStartRecording.Visible = false;
            this.recordToolStripMenuItem.Visible = false;
            this.tcSource.Selected += new TabControlEventHandler(tcSource_SelectedTabChanged);
            this.chbDiagnostic.CheckedChanged += new EventHandler(chbDiagnostic_CheckedChanged);
            this.pictureBoxTracker.MouseLeave += pictureBoxTracker_MouseLeave;
            this.pictureBoxRealTracking.MouseLeave += pictureBoxTracker_MouseLeave;

            BackProjectionForm.SetParent(this);

            #region MenuItems Event Handlers
            menuInner320x240.CheckedChanged += new EventHandler(menuInner_CheckedChanged);
            menuInner640x480.CheckedChanged += new EventHandler(menuInner_CheckedChanged);
            menuInnerFirstAid.CheckedChanged += new EventHandler(menuInner_CheckedChanged);
            menuInnerBP.CheckedChanged += new EventHandler(menuInner_CheckedChanged);
            menuTrackingDetails.CheckedChanged += new EventHandler(menuInner_CheckedChanged);
            this.menuItemSimpleSize.CheckedChanged += new EventHandler(menuItemSimpleSize_CheckedChanged);
            this.menuItemSimpleSize.Click += (sender, e) => menuItemSimpleSize.Checked = !menuItemSimpleSize.Checked;
            #endregion


            #endregion //end obsluzne metody

            #region Validations
            //pri vyberu cile neprobehne validace posledniho zmeneneho textoveho pole histogramu 
            this.pictureBoxTracker.MouseEnter += (sender, e) => {
                this.Validate();
            };
            this.txeRedBins.Validating += new CancelEventHandler(txeRedBins_Validating);
            this.txeGreenBins.Validating += new CancelEventHandler(txeGreenBins_Validating);
            this.txeBlueBins.Validating += new CancelEventHandler(txeBlueBins_Validating);
            #endregion //end validations

            #region Tooltips
            this.txeScaleRatio.MouseHover += (sender, e) =>
                this._toolTip.SetToolTip(sender as Control, "0 = disable adaptive scale");
            this.txeTimerInterval.MouseHover += (sender, e) =>
                this._toolTip.SetToolTip(sender as Control, "Shortcut: [Up] - Increase  [Down] - Decrease");
            this.btnOpen.MouseHover += (sender, e) =>
                this._toolTip.SetToolTip(sender as Control, "Shortcut: [O] ");
            this.btnStart.MouseHover += (sender, e) =>
                this._toolTip.SetToolTip(sender as Control, "Shortcut: Start / Stop  [Space] or [Pause]");
            this.btnResetSequence.MouseHover += (sender, e) =>
                this._toolTip.SetToolTip(sender as Control, "Shortcut: [R]");
            this.txtTopLeft.MouseHover += (sender, e) =>
                this._toolTip.SetToolTip(sender as Control, "X-TopLeft ,Y-TopLeft ; X-BottomRight ,Y-BottomRight");
            #endregion // end tooltips

            #region Shortcuts
            this.KeyDown += new KeyEventHandler(CAMShiftForm_KeyDown);
            this.btnOpen.KeyDown += (sender, e) => {
                this.CAMShiftForm_KeyDown(sender, e);
            };
            this.btnResetSequence.KeyDown += (sender, e) => {
                this.CAMShiftForm_KeyDown(sender, e);
            };
            this.btnStart.KeyDown += (sender, e) => {
                this.CAMShiftForm_KeyDown(sender, e);
            };
            this.gbHistoramSize.KeyDown += (sender, e) => {
                this.CAMShiftForm_KeyDown(sender, e);
            };
            this.txeTimerInterval.KeyDown += (sender, e) => {
                this.CAMShiftForm_KeyDown(sender, e);
            };
            this.txeScaleRatio.KeyDown += (sender, e) => {
                this.CAMShiftForm_KeyDown(sender, e);
            };
            #endregion //end shortcuts
        }


        #region Event Handlers

        /// <summary>
        /// Obsluzna metoda udalosti pri kliknuti na tlacitko Reload, provede
        /// znovu nacteni sekvence a zobrazi jeji prvni snimek na pictureBoxu
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnReload_Click(object sender, EventArgs e)
        {
            if (this.chbDiagnostic.Checked) {
                this._diagnostic.Save(this.GetDiagnosticName(), this._area);
            }
            this.ResetSequence();
        }

        /// <summary>
        /// Umozní výber adresare se sekvenci snímku k analyzovani.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnOpen_Click(object sender, EventArgs e)
        {
            this.errorProvider.SetError(this.btnOpen, string.Empty);
            this._imageCounter = 0;
            this._endSequence = false;
            FolderBrowserDialog fbDlg = new FolderBrowserDialog();
            fbDlg.SelectedPath = Settings.Default.Path;
            if (fbDlg.ShowDialog() == DialogResult.OK) {
                this._path = fbDlg.SelectedPath + @"\";
                if ((Directory.GetFiles(this._path).Length) == 0) {
                    this.errorProvider.SetError(this.btnOpen, "Vybraný adresář je prázdný");
                    return;
                }
                this.LoadSingleImage();
                Settings.Default.Path = this._path;
                Settings.Default.Save();

                string[] sequenceName = this._path.Split('\\');
                if (sequenceName.Length - 2 >= 0) {
                    this._sequenceName = sequenceName[sequenceName.Length - 2].ToUpper();
                }
                this.SetTargetSelectionEvents(this.pictureBoxTracker);
                TrackingDetailsForm.Instance.Clear();
            }
        }

        /// <summary>
        /// Oznaci cil podle vyplnych souradnic v textovem poli
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btnMark_Click(object sender, EventArgs e)
        {
            if (this.txtTopLeft.Text == string.Empty) {
                this.selection_OnTargetSelected(null, new TargetSelectionEventArgs(null, new Rectangle(this._area.ObjectArea.UpperLeftCorner,
                    new Size(_area.ObjectArea.Width, _area.ObjectArea.Height))));

            }
            string[] points = this.txtTopLeft.Text.Split(';');
            Point markLeft = this.GetPoint(points[0]);
            Point markRight = this.GetPoint(points[points.Length - 1]);
            Size s = new Size();
            s.Height = markRight.Y - markLeft.Y;
            s.Width = markRight.X - markLeft.X;
            this.selection_OnTargetSelected(null, new TargetSelectionEventArgs(null, new Rectangle(markLeft, s)));
        }
        /// <summary>
        /// Handles the MouseLeave event of the pictureBoxTracker control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void pictureBoxTracker_MouseLeave(object sender, EventArgs e)
        {

            lblX.Visible = lblY.Visible = lblXValue.Visible = lblYValue.Visible = false;
        }
        /// <summary>
        /// Handles the CheckedChanged event of the menuItemSimpleSize control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void menuItemSimpleSize_CheckedChanged(object sender, EventArgs e)
        {
            if (this._meanShift != null) {
                this._meanShift.IsSimpleSizeCalculation = menuItemSimpleSize.Checked;
            }
            Settings.Default.IsSimpleSizeCalc = menuItemSimpleSize.Checked;
        }

        /// <summary>
        /// Handles the SizeChanged event of the pictureBoxRealTracking control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void pictureBoxRealTracking_SizeChanged(object sender, EventArgs e)
        {
            this.ResizeForm();
        }

        /// <summary>
        /// Handles the CheckedChanged event of the chbDiagnostic control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void chbDiagnostic_CheckedChanged(object sender, EventArgs e)
        {
            this.txtTopLeft.Visible = this.btnMark.Visible = this.chbDiagnostic.Checked;
            if (chbDiagnostic.Checked && _diagnostic != null) {
                string path = OpenDiagnosticDestinationPathDialog();

                _diagnostic.Path = path;
            }
        }

        /// <summary>
        /// Handles the CheckedChanged event of the menuInner control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void menuInner_CheckedChanged(object sender, EventArgs e)
        {
            if (sender.GetType().Equals(menuInner640x480.GetType()) || sender.GetType().Equals(menuInner320x240.GetType())) {
                FrameCapturer.IsDefaultResoulutionSelected = this.menuInner320x240.Checked;
            }
            ToolStripMenuItem menuItem = sender as ToolStripMenuItem;
            if (menuItem == null)
                return;
            if (menuItem == menuInner320x240) {
                this.menuInner640x480.Checked = !menuItem.Checked;
            } else if (menuItem == menuInner640x480) {
                this.menuInner320x240.Checked = !menuItem.Checked;
            } else if (menuItem == menuInnerFirstAid) {
                this.IsFirstAidSelected = menuItem.Checked;
            } else if (menuItem == menuTrackingDetails) {
                if (this.menuTrackingDetails.Checked && this.IsPlaying) {
                    TrackingDetailsForm.Instance.Show();
                } else if (TrackingDetailsForm.Instance.IsLoaded) {
                    TrackingDetailsForm.Instance.Close();
                }
            } else if (this.menuInnerBP == menuItem) {
                if (this.menuInnerBP.Checked && this.IsPlaying) {
                    BackProjectionForm.Instance.Show();
                } else if (BackProjectionForm.Instance.IsLoaded) {
                    BackProjectionForm.Instance.Close();
                }
            }
        }
        /// <summary>
        /// Handles the MouseMove event of the pictureBoxRealTracking control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Forms.MouseEventArgs"/> instance containing the event data.</param>
        void pictureBoxRealTracking_MouseMove(object sender, MouseEventArgs e)
        {
            SetPositionLabels(e);
            if (e.Button == MouseButtons.Left && this._selection != null) {
                int x = this._selection.StartPosition.X;
                int y = this._selection.StartPosition.Y;
                Graphics g = (sender as PictureBox).CreateGraphics();
                g.DrawRectangle(this._penObject, x, y, e.X - x, e.Y - y);
            }
        }

        private void SetPositionLabels(MouseEventArgs e)
        {
            if (!lblX.Visible || !lblY.Visible) {
                lblX.Visible = lblY.Visible = lblXValue.Visible = lblYValue.Visible = true;
            }
            lblXValue.Text = e.X.ToString();
            lblYValue.Text = e.Y.ToString();

        }

        /// <summary>
        /// Obslužná metoda MouseMove, se stara o graficke oznaceni cile pri tahnuti mysi -
        /// plynule vykreslovani sledovaciho okna pri vyberu.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void pictureBox1_MouseMove(object sender, MouseEventArgs e)
        {
            SetPositionLabels(e);
            if (e.Button == MouseButtons.Left && this._selection != null) {
                this.LoadSingleImage();
                int x = this._selection.StartPosition.X;
                int y = this._selection.StartPosition.Y;
                using (Graphics g = Graphics.FromImage(this.pictureBoxTracker.Image)) {
                    g.DrawRectangle(this._penObject, x, y, e.X - x, e.Y - y);
                }
            }
        }

        /// <summary>
        /// Provadi reziji na pozadi mezi prepinani jednotlivych zalozek, spousti a vypina kamerove zarizeni, presmerovava tok snimku z kamery a pod.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void tcSource_SelectedTabChanged(object sender, EventArgs e)
        {
            if (this.tcSource.SelectedTab == tpRealTimeOT) {
                this._isRealTimeSelected = true;
                if ((bool)this.btnStart.Tag)
                    this.Stop();
                if (this._frameCapturer != null) {
                    this._frameCapturer.ChangeResolution(this.menuInner320x240.Checked);
                }
                this._frameCapturer = new FrameCapturer();
                this._frameCapturer.DeviceFound += new EventHandler(m_frameCapturer_DeviceFound);
                if (this._frameCapturer.IsDeviceSet)
                    this._frameCapturer.Start();
                this._frameCapturer.FrameCaptured += new FrameCapturer.FrameCaptureEventHandler(m_frameCapturer_FrameCaptured);
                Thread.Sleep(500); // cekani na ziskani prvniho snimku kamery
                this.SetTargetSelectionEvents(this.pictureBoxRealTracking);
                this.btnOpen.Enabled = this.btnResetSequence.Enabled = false;
            } else {
                BufferManager.Instance.Reset();
                this._isRealTimeSelected = false;
                if (this._frameCapturer != null) {
                    this._frameCapturer.Stop();
                    this._frameCapturer.FrameCaptured -= new FrameCapturer.FrameCaptureEventHandler(m_frameCapturer_FrameCaptured);
                }
                this.btnOpen.Enabled = this.btnResetSequence.Enabled = true;
            }
        }

        /// <summary>
        /// Handles the DeviceFound event of the m_frameCapturer control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void m_frameCapturer_DeviceFound(object sender, EventArgs e)
        {
            DevicesEventArgs dEvent = e as DevicesEventArgs;
            if (dEvent.Devices.Count > 0) {
                if (dEvent.Devices.Count > 1) {
                    DeviceDetectedForm detectForm = new DeviceDetectedForm(dEvent.Devices);
                    detectForm.DeviceSelected += (devSender, devE) =>
                        this._frameCapturer.SetDevice(((SelectedDeviceEventArgs)devE).SelectedIndex);
                    detectForm.ShowDialog(this);
                } else
                    this._frameCapturer.SetDevice(0);
                this._frameCapturer.Start();

            } else
                MessageBox.Show("Nerozpoznána žádná kamera", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            this._frameCapturer.DeviceFound -= new EventHandler(m_frameCapturer_DeviceFound);

        }
        /// <summary>
        /// Odchytavani jednotlivych snimku a nastavovani velikosti picture boxu.
        /// Posilani snimku z kamery do fronty.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="AForge.Video.NewFrameEventArgs"/> instance containing the event data.</param>
        void m_frameCapturer_FrameCaptured(object sender, Bitmap e)
        {
            if (!this._isRealTrackingRunning) {
                this.SetPictureBox(this.pictureBoxRealTracking, e.Clone() as Bitmap);
            } else {
                BufferManager.Instance.Add(e.Clone() as Bitmap);
                this._imageCounter++;
            }
        }

        /// <summary>
        /// Obslužná metoda události na zmacknuti klavesy na formulari
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void CAMShiftForm_KeyDown(object sender, KeyEventArgs e)
        {
            switch (e.KeyCode) {
                case Keys.Pause:
                    this.btnStart_Click(sender, e);
                    break;
                case Keys.R:
                    this.btnReload_Click(sender, e);
                    break;
                case Keys.O:
                    this.btnOpen_Click(sender, e);
                    break;
                case Keys.Down:
                    this.DecreaseSpeed();
                    break;
                case Keys.Up:
                    this.IncreaseSpeed();
                    break;
                default:
                    return;
            }
        }
        #endregion

        #region Methods

        /// <summary>
        /// Navšení intervalu timeru o +1
        /// </summary>
        private void IncreaseSpeed()
        {
            int interval = (Convert.ToInt32(this.txeTimerInterval.Text));
            this.txeTimerInterval.Text = (++interval).ToString();
        }
        /// <summary>
        /// Snižení intervalu timeru o -1
        /// </summary>
        private void DecreaseSpeed()
        {
            int interval = (Convert.ToInt32(this.txeTimerInterval.Text));
            this.txeTimerInterval.Text = (--interval).ToString();
        }

        #region Settings methods

        /// <summary>
        /// Ulozeni Rozmeru formulare
        /// </summary>
        private void SaveFormSize()
        {
            Settings.Default.FormWidth = this.Width;
            Settings.Default.FormHeight = this.Height;
        }
        /// <summary>
        /// Ulozi informace o viditelnosti seznamu informaci
        /// </summary>
        /// <param name="value"></param>
        private void SaveInformationsVisibility()
        {
            Settings.Default.InformationsVisibility = this.menuTrackingDetails.Checked;
        }
        private void SaveBpVisibility()
        {
            Settings.Default.IsBpVisible = this.menuInnerBP.Checked;
        }
        /// <summary>
        /// Uloyi hodnoty rychlosti odesilani obrazku a pomeru adaptace stran
        /// </summary>
        private void SaveSendingSpeed()
        {
            int speed = 0;
            int ratio = 0;
            Int32.TryParse(this.txeTimerInterval.Text, out speed);
            Int32.TryParse(this.txeScaleRatio.Text, out ratio);
            Settings.Default.SendingSpeed = speed;
            Settings.Default.AdaptiveRatio = ratio;

        }
        /// <summary>
        /// Ulozi hodnoty velikosti RGB histogramu
        /// </summary>
        private void SaveRGBValues()
        {
            string rgb = String.Format("{0},{1},{2}", this._redBins, this._greenBins, this._blueBins);
            Settings.Default.RGBHist = rgb;
        }
        /// <summary>
        /// Uložení typu histogramu
        /// </summary>
        private void SaveHistogramType()
        {
            Settings.Default.HistogramType = (int)this.cbHistogramType.SelectedItem;
        }
        /// <summary>
        /// Ulozeni cesty
        /// </summary>
        private void SavePath()
        {
            Settings.Default.Path = this._path;
        }
        /// <summary>
        /// Ulozi nastaveni rozliseni 
        /// </summary>
        private void SaveResolution()
        {
            Settings.Default.IsDefaultResolution = this.menuInner320x240.Checked;
        }
        private void SaveAdaptation()
        {
            Settings.Default.SelfAdaptation = this.menuInnerFirstAid.Checked;
        }
        private void SaveTracker()
        {
            Settings.Default.Tracker = this.menuComboBoxTrackers.SelectedIndex;
        }

        /// <summary>
        /// Uložení všech nastavení
        /// </summary>
        private void SaveSettings()
        {
            SaveTracker();
            SaveFormSize();
            SaveInformationsVisibility();
            SaveBpVisibility();
            SaveSendingSpeed();
            SaveRGBValues();
            SaveHistogramType();
            SavePath();
            SaveResolution();
            SaveAdaptation();
            Settings.Default.Save();
        }
        /// <summary>
        /// Nacteni Settings
        /// </summary>
        private void LoadSettings()
        {
            if (Settings.Default.FormHeight != 0 && Settings.Default.FormWidth != 0) {
                this.Height = Settings.Default.FormHeight;
                this.Width = Settings.Default.FormWidth;
            }
            //tracking details visibility
            this.menuTrackingDetails.Checked = Settings.Default.InformationsVisibility;
            //back projection visibility
            this.menuInnerBP.Checked = Settings.Default.IsBpVisible;
            if (Settings.Default.SendingSpeed != 0) {
                this.txeTimerInterval.Text = Settings.Default.SendingSpeed.ToString();
                this.SenderInterval = Settings.Default.SendingSpeed;
            }

            if (Settings.Default.AdaptiveRatio != 0) {
                this._scaleRatio = Settings.Default.AdaptiveRatio;
                this.txeScaleRatio.Text = this._scaleRatio.ToString();
            }
            if (!String.IsNullOrEmpty(Settings.Default.RGBHist)) {
                string[] s = Settings.Default.RGBHist.Split(',');
                if (s.Length == 3) {
                    this.txeRedBins.Text = s[0];
                    this.txeGreenBins.Text = s[1];
                    this.txeBlueBins.Text = s[2];
                    Int32.TryParse(this.txeRedBins.Text, out this._redBins);
                    Int32.TryParse(this.txeGreenBins.Text, out this._greenBins);
                    Int32.TryParse(this.txeBlueBins.Text, out this._blueBins);
                }
            }
            this.cbHistogramType.SelectedItem = (HistogramType)Settings.Default.HistogramType;
            this._path = Settings.Default.Path;
            //resolution
            this.menuInner320x240.Checked = Settings.Default.IsDefaultResolution;
            //adaptation
            this.menuInnerFirstAid.Checked = this.IsFirstAidSelected = Settings.Default.SelfAdaptation;
            //tracker
            this.menuComboBoxTrackers.SelectedIndex = Settings.Default.Tracker;
            //simple target size calculatiom
            this.menuItemSimpleSize.Checked = Settings.Default.IsSimpleSizeCalc;

        }
        #endregion //endSettings

        #region Validations
        /// <summary>
        /// Validace textboxu pro B rozmer histogramu
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void txeBlueBins_Validating(object sender, CancelEventArgs e)
        {
            int value = this.SetValidValue((TextBox)sender);
            if (value == 0)
                e.Cancel = true;
            else
                this._blueBins = value;
        }
        /// <summary>
        /// Validace textboxu pro G rozmer histogramu
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void txeGreenBins_Validating(object sender, CancelEventArgs e)
        {
            int value = this.SetValidValue((TextBox)sender);
            if (value == 0)
                e.Cancel = true;
            else
                this._greenBins = value;
        }
        /// <summary>
        /// Validace textboxu pro R rozmer histogramu
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void txeRedBins_Validating(object sender, CancelEventArgs e)
        {
            int value = this.SetValidValue((TextBox)sender);
            if (value == 0)
                e.Cancel = true; //zamezeni opusteni pole
            else
                this._redBins = value;
        }
        #endregion//end Validations
        /// <summary>
        /// Adaptace velikosti formulaře podle velikosti zobrazovaneho obrazku v pictureBoxu
        /// </summary>
        private void ResizeForm()
        {
            this.tcSource.BeginInvoke((MethodInvoker)delegate {
                if (this.IsRealTimeSelected) {
                    if (pictureBoxRealTracking.Image != null) {
                        this.tcSource.Width = this.pictureBoxRealTracking.Image.Width + DEFAULT_PADDING;
                        this.tcSource.Height = this.pictureBoxRealTracking.Image.Height + DEFAULT_PADDING;
                    } else {
                        this.tcSource.Width = this.pictureBoxRealTracking.Width + DEFAULT_PADDING;
                        this.tcSource.Height = this.pictureBoxRealTracking.Height + DEFAULT_PADDING;
                    }
                    this.Height = this.pictureBoxRealTracking.Bottom + DEFAULT_PADDING * 6;
                    this.Width -= (this.gbControl.Left - this.pictureBoxRealTracking.Right) - DEFAULT_PADDING * 2;
                } else {
                    if (this.pictureBoxTracker.Image != null) {
                        this.tcSource.Width = this.pictureBoxTracker.Image.Width + DEFAULT_PADDING;
                        this.tcSource.Height = this.pictureBoxTracker.Image.Height + DEFAULT_PADDING;
                    } else {
                        this.tcSource.Width = this.pictureBoxTracker.Width + DEFAULT_PADDING;
                        this.tcSource.Height = this.pictureBoxTracker.Height + DEFAULT_PADDING;
                    }
                    this.Height = this.pictureBoxTracker.Bottom + DEFAULT_PADDING * 6;
                    this.Width -= (this.gbControl.Left - this.pictureBoxTracker.Right) - DEFAULT_PADDING * 2;
                }
            });
        }

        /// <summary>
        /// Vrati validni hodnotu, pokud neni hodnota validni, nastavi error provider k predanemu
        /// controlu a vrati 0.
        /// </summary>
        /// <param name="textBox"></param>
        /// <returns></returns>
        private int SetValidValue(TextBox textBox)
        {
            int value = 0;
            this.errorProvider.SetError(textBox, string.Empty);
            if (Int32.TryParse(textBox.Text, out value)) {
                value = Convert.ToInt32(textBox.Text);
                value = (value > 0) ? value : 0;
                if (value == 0)
                    this.errorProvider.SetError(textBox, "Velikost nesmí nabývat nulových nebo záporných hodnot.");
            } else {
                this.errorProvider.SetError(textBox, "Zadaná hodnota není číslo");
            }
            return value;
        }
        /// <summary>
        /// Inicializuje vychozí hodnoty TextBoxu pro velikost histogramu 
        /// </summary>
        private void InitializeHistogramTextBoxes()
        {
            if (String.IsNullOrEmpty(this.txeRedBins.Text))
                this.txeRedBins.Text = R.ToString();
            if (String.IsNullOrEmpty(this.txeGreenBins.Text))
                this.txeGreenBins.Text = G.ToString();
            if (String.IsNullOrEmpty(this.txeBlueBins.Text))
                this.txeBlueBins.Text = B.ToString();
        }
        /// <summary>
        /// Znovu nacte obrazek z disku a umisti jej na picturebox, podle poradi aktualniho obrazku
        /// Metoda slouzi jako vypomoc pri vykreslovani okna inicializacni Cilove oblasti. Z duvodu,
        /// ze obrazky v pameti jsou jiz pokreslene a ja potrebuju vzdy cisty, aby byla dobre viditelna
        /// graficka stopa pri tahnuti mysi - pri inicializaci cile.
        /// </summary>
        private void LoadSingleImage()
        {
            int counter = this._imageCounter;
            Bitmap loadedImage;
            //muze nastat pokud soubory nejsou cislovany od nuly
            if (File.Exists(this._path + (counter).ToString() + FILENAME_EXTENSION)) {
                loadedImage = (Image.FromFile(this._path + (counter).ToString() + FILENAME_EXTENSION) as Bitmap);
            } else {
                loadedImage = (Image.FromFile(this._path + (++counter).ToString() + FILENAME_EXTENSION) as Bitmap);
            }
            this.SetPictureBox(this.pictureBoxTracker, loadedImage);
        }
        /// <summary>
        /// Obslužná metoda udalosti na změnu textu Textboxu s pomerem adaptivniho okna.
        /// Zamezuje vepsani nenumerickych znaku do pole. V tomto pripade do pole
        /// vklada vychozi hodnotu.
        /// </summary>
        /// <param name="sender">sender</param>
        /// <param name="e">event args</param>
        void txeScaleRatio_TextChanged(object sender, EventArgs e)
        {
            int result = 0;
            if (!Int32.TryParse(this.txeScaleRatio.Text, out result)) {
                this.txeTimerInterval.Text = DEFAULT_RATIO;
                return;
            } else {
                if (this._meanShift != null)
                    this._meanShift.ScaleRatio = result;
            }
        }
        /// <summary>
        /// Umoznuje zmenu rychlosti odesilani snimku k analyze za behu aplikace
        /// </summary>
        /// <param name="sender">sender</param>
        /// <param name="e">event args</param>
        void textBox1_TextChanged(object sender, EventArgs e)
        {
            if (String.IsNullOrEmpty(this.txeTimerInterval.Text))
                return;
            this.SenderInterval = this.SetValidValue((TextBox)sender);
            this.btnStart_Click(sender, e);
            this.btnStart_Click(sender, e);
        }
        /// <summary>
        /// Metoda zasílá obrazky k mean - shift analyze podle poradi z disku
        /// </summary>
        /// <param name="imageNo"></param>
        private void SendImageToAnalyzeFromDrive()
        {
            Bitmap loadedImage;
            if (File.Exists(this._path + (++this._imageCounter).ToString() + FILENAME_EXTENSION)) {
                loadedImage = (Image.FromFile(this._path + (this._imageCounter).ToString() + FILENAME_EXTENSION) as Bitmap);
            } else {
                this._endSequence = true;
                this.Stop();
                return;
            }
            this._meanShift.Tracking(ref loadedImage);
        }
        /// <summary>
        /// Odesila obrazky k trackovani z fronty - pouziti u real time trackovani
        /// </summary>
        private void SendImageToAnalyzeFromQueue()
        {
            if (!BufferManager.Instance.IsEmpty) {
                if (this._meanShift != null) {
                    Bitmap bmp = BufferManager.Instance.GetFrame();

                    this._meanShift.Tracking(ref bmp);
                }
            }
        }
        /// <summary>
        /// Metoda zasle obrazek k MS analyze z ruzneho umisteni, pokud bylo zvoleno nacitani do pameti
        /// odesila obrazky ze seznamu, jinak primo z umisteni na disku
        /// </summary>
        private void SendImageToAnalyze()
        {
            if ((this._images == null || this._images.Count == 0) && !this._isRealTrackingRunning)
                this.SendImageToAnalyzeFromDrive();
            else if (this._isRealTrackingRunning) {
                this.SendImageToAnalyzeFromQueue();
            }
        }

        /// <summary>
        /// Obslužná metoda události OnTargetSelected, vyvolá se , když je vybrán sledovaný objekt.
        /// Vykresli se ohraniceni kolem hledaného cíle, volá se konstruktor Sledovaciho algoritmu
        /// a registruje se udalost na nalezení cíle
        /// </summary>
        /// <param name="sender">TargetSelection</param>
        /// <param name="e">TargetSelectionEventArgs</param>
        void selection_OnTargetSelected(TargetSelection sender, TargetSelectionEventArgs e)
        {
            AdaptiveTargetArea ata = new AdaptiveTargetArea(new Point(e.TargetRectangle.X, e.TargetRectangle.Y),
                new Point(e.TargetRectangle.Right, e.TargetRectangle.Bottom));
            this._area = ata;
            if (!this.IsRealTimeSelected) {
                InitializeTracker(this.pictureBoxTracker, ata);
            } else {
                this._isRealTrackingRunning = true;
                this.txeTimerInterval.Text = "1";
                InitializeTracker(this.pictureBoxRealTracking, ata);
                this.btnStart_Click(this, new EventArgs());
            }
        }
        /// <summary>
        ///  Inicializace vybraneho trackeru
        /// </summary>
        /// <param name="pictureBox">The picture box.</param>
        /// <param name="ata">The ata.</param>
        private void InitializeTracker(PictureBox pictureBox, AdaptiveTargetArea ata)
        {
            Graphics g = pictureBox.CreateGraphics();
            g.DrawRectangle(this._penObject, ata.ObjectArea.AreaRectangle);
            Bitmap bmp = (pictureBox.Image as Bitmap);
            //ImageManager.SaveImagesSet(pictureBox.Image as Bitmap, null, new Size());
            HistogramType type = (HistogramType)this.cbHistogramType.SelectedItem;
            if (this.menuComboBoxTrackers.SelectedIndex == 0) {
                this._meanShift = new MeanShift(this._redBins, this._greenBins, this._blueBins, 25, type, ref bmp, ata);
            } else if (this.menuComboBoxTrackers.SelectedIndex == 2)
                this._meanShift = new CAMShiftDM(this._redBins, this._greenBins, this._blueBins, 25, type, ref bmp, ata);
            else
                this._meanShift = new CAMShift(this._redBins, this._greenBins, this._blueBins, 25, type, ref bmp, ata);
            this._meanShift.IsSimpleSizeCalculation = this.menuItemSimpleSize.Checked;
            this._meanShift.ScaleRatio = this._scaleRatio;
            this._meanShift.TargetFound -= new MeanShift.TrackerHandler(TargetFound);
            this._meanShift.TargetFound += new MeanShift.TrackerHandler(TargetFound);
            this._meanShift.EnableDiagnosticMode = this.chbDiagnostic.Checked;
        }
        /// <summary>
        /// Obslužná metoda udalosti - TargetFound, vyvolava se pri nalezeni pozice cíle,
        /// zde slouzi k vykresleni pozice sledovaneho objektu + predani konkretnich informaci do
        /// seznamu.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void TargetFound(object sender, TrackerEventArgs e)
        {
            if (!this.IsRealTimeSelected)
                this.IdentifyTarget(this.pictureBoxTracker, e);
            else
                this.IdentifyTarget(this.pictureBoxRealTracking, e);
            string info = String.Format("{0}  ImageNo: {1}     Iterations {2}     Duration: {3}   ms", e.TargetArea.ObjectArea.Center,
                _imageCounter, e.MeanshiftIterations, e.ComputationTime);
            if (this.menuTrackingDetails.Checked && !TrackingDetailsForm.Instance.IsLoaded) {
                TrackingDetailsForm.Instance.Show();
                TrackingDetailsForm.Instance.Dock(this);
            }
            TrackingDetailsForm.Instance.AddRecord(info);

            if (this.chbDiagnostic.Checked) {
                this._diagnostic.AddPositionSet(this.pictureBoxTracker.PointToClient(MousePosition), e.TargetArea.Center);
                this._diagnostic.AddEllapsedTime(e.ComputationTime);
                this._diagnostic.AddIteration(e.MeanshiftIterations);
                this._diagnostic.AddTargetArea(e.TargetArea);
            }
        }

        /// <summary>
        /// Oznaceni cile na obrazcich a nastaveni pictureBoxu
        /// </summary>
        /// <param name="e"></param>
        private void IdentifyTarget(PictureBox pictureBox, TrackerEventArgs e)
        {
            using (Graphics gr = Graphics.FromImage(e.CalculatedBitmap as Image)) {
                this.SetPictureBox(pictureBox, e.CalculatedBitmap);
                if (!e.TargetArea.ObjectArea.AreaRectangle.Height.Equals(AdaptiveTargetArea.FIRST_AID.Height - 1)) {
                    gr.DrawRectangle(this._penObject, e.TargetArea.ObjectArea.AreaRectangle);
                    gr.DrawLine(this._penObject, new Point(e.TargetArea.Center.X, e.TargetArea.ObjectArea.AreaRectangle.Top),
                        new Point(e.TargetArea.Center.X, e.TargetArea.ObjectArea.AreaRectangle.Bottom));
                    gr.DrawLine(this._penObject, new Point(e.TargetArea.Center.X - e.TargetArea.ObjectArea.Width / 2, e.TargetArea.ObjectArea.Center.Y),
                        new Point(e.TargetArea.ObjectArea.AreaRectangle.Right, e.TargetArea.ObjectArea.Center.Y));
                    //g.DrawLine(this.m_penObject, new Point(e.TargetArea.Center.X, this.pictureBoxTracker.Top),
                    //    new Point(e.TargetArea.Center.X, this.pictureBoxTracker.Bottom));
                    //g.DrawLine(this.m_penObject, new Point(this.pictureBoxTracker.Left, e.TargetArea.ObjectArea.Center.Y),
                    //    new Point(this.pictureBoxTracker.Right, e.TargetArea.ObjectArea.Center.Y));
                }
            }
            if (e is CAMShiftEventArgs) {
                CAMShiftEventArgs camE = e as CAMShiftEventArgs;
                using (Graphics gs = Graphics.FromImage(camE.BackProjection as Image)) {
                    if (!camE.TargetArea.ObjectArea.AreaRectangle.Height.Equals(AdaptiveTargetArea.FIRST_AID.Height - 1))
                        gs.DrawRectangle(this._penObjectBP, e.TargetArea.ObjectArea.AreaRectangle);
                    if (this.menuInnerBP.Checked && !BackProjectionForm.Instance.IsLoaded) {
                        BackProjectionForm.Instance.Show();
                        BackProjectionForm.Instance.DockWindow();
                    }
                    BackProjectionForm.Instance.SetImage(camE.BackProjection);
                    DistributionData data = camE.Data;
                    //Save image for camshift 
                    if (SAVE_IMAGES) {
                        ImageManager.SaveImagesSet(camE.CalculatedBitmap, camE.BackProjection, camE.CalculatedBitmap.Size, null);
                    }

                    #region Orientation

                    ////orientation
                    //        using (Graphics g = Graphics.FromImage(camE.CalculatedBitmap as Image)) {

                    //            Pen penLength = new Pen(Color.Azure, 2);
                    //            Pen penWidth = new Pen(Color.Navy, 2);


                    //            double l = data.Length;
                    //            double w = data.Width;

                    //            //ROLL -90 to +90
                    //            float rollAngleDeg = (float)data.AngleToDegrees((float)data.AngleInRadians);
                    //            float rollAngleRad = (float)data.AngleInRadians;

                    //            if (rollAngleDeg < 0) //4th quadrant
                    //{
                    //                rollAngleDeg = -1 * rollAngleDeg;
                    //                g.DrawLine(penLength, camE.TargetArea.Center, new Point(camE.TargetArea.Center.X + (int)Math.Ceiling(l * Math.Cos(data.AngleToRadians(rollAngleDeg))),
                    //                    camE.TargetArea.Center.Y + (int)Math.Ceiling(l * Math.Sin(data.AngleToRadians(rollAngleDeg))))); //centroid to top with l
                    //                g.DrawLine(penLength, camE.TargetArea.Center, new Point(camE.TargetArea.Center.X - (int)Math.Ceiling(l * Math.Cos(data.AngleToRadians(rollAngleDeg))),
                    //                    camE.TargetArea.Center.Y - (int)Math.Ceiling(l * Math.Sin(data.AngleToRadians(rollAngleDeg))))); //centroid to bottom with l
                    //                g.DrawLine(penWidth, camE.TargetArea.Center,
                    //                    new Point(camE.TargetArea.Center.X - (int)Math.Ceiling(w * Math.Cos(data.AngleToRadians(90.0f - rollAngleDeg))),
                    //                    camE.TargetArea.Center.Y + (int)Math.Ceiling(w * Math.Sin(data.AngleToRadians(90.0f - rollAngleDeg))))); //centroid to right with w
                    //                g.DrawLine(penWidth, camE.TargetArea.Center,
                    //                    new Point(camE.TargetArea.Center.X + (int)Math.Ceiling(w * Math.Cos(data.AngleToRadians(90.0f - rollAngleDeg))),
                    //                    camE.TargetArea.Center.Y - (int)Math.Ceiling(w * Math.Sin(data.AngleToRadians(90.0f - rollAngleDeg))))); //centroid to left with w            
                    //            } else //1st quadrant
                    //{
                    //                g.DrawLine(penLength, camE.TargetArea.Center, new Point(camE.TargetArea.Center.X + (int)Math.Ceiling(l * Math.Cos(rollAngleRad)),
                    //                                        camE.TargetArea.Center.Y - (int)Math.Ceiling(l * Math.Sin(rollAngleRad)))); //centroid to top with l
                    //                g.DrawLine(penLength, camE.TargetArea.Center, new Point(camE.TargetArea.Center.X - (int)Math.Ceiling(l * Math.Cos(rollAngleRad)),
                    //                                        camE.TargetArea.Center.Y + (int)Math.Ceiling(l * Math.Sin(rollAngleRad)))); //centroid to bottom with l
                    //                float angle = 90.0f - rollAngleDeg;
                    //                if (w != double.NaN) {

                    //                    g.DrawLine(penWidth, camE.TargetArea.Center,
                    //                                            new Point(camE.TargetArea.Center.X + (int)Math.Ceiling(w * Math.Cos(data.AngleToRadians(angle))),
                    //                                            camE.TargetArea.Center.Y + (int)Math.Ceiling(w * Math.Sin(data.AngleToRadians(angle))))); //centroid to right with w
                    //                    g.DrawLine(penWidth, camE.TargetArea.Center,
                    //                                            new Point(camE.TargetArea.Center.X - (int)Math.Ceiling(w * Math.Cos(data.AngleToRadians(angle))),
                    //                                            camE.TargetArea.Center.Y - (int)Math.Ceiling(w * Math.Sin(data.AngleToRadians(angle))))); //centroid to left with w
                    //                }
                    //            }
                    //        } 
                    #endregion
                }

            } else {
                //save image for non camshift
                if (SAVE_IMAGES) {
                    ImageManager.SaveImagesSet(e.CalculatedBitmap, null, e.CalculatedBitmap.Size, e.TargetArea);
                }

            }
        }
        /// <summary>
        /// Nastaví daný picure box - velikost a obrazek podle predane bitmapy
        /// </summary>
        /// <param name="pictBox"></param>
        /// <param name="bmp"></param>
        private void SetPictureBox(PictureBox pictBox, Bitmap bmp)
        {
            if (bmp == null)
                return;
            if (pictBox.InvokeRequired) {
                try {
                    pictBox.Invoke(new MethodInvoker(() => {
                        pictBox.Size = new Size(bmp.Width, bmp.Height);
                        tcSource.Size = new Size(bmp.Width + tcSource.Margin.Right + tcSource.Margin.Left + 20,
                            bmp.Height + tcSource.Margin.Top + tcSource.Margin.Bottom + 30);
                    }
                    ));
                } catch (ObjectDisposedException ex) {
                    Debug.Write(ex.Message);
                    Trace.Write(ex.StackTrace);
                    return;
                }
            }
            pictBox.Invoke(new MethodInvoker(() => {
                if (pictBox.Size != bmp.Size) {
                    pictBox.Size = new Size(bmp.Width, bmp.Height);
                    this.ResizeForm();
                }
            }));
            lock (pictBox) {
                pictBox.Image = bmp;
            }

        }


        /// <summary>
        /// Nastavi soucasny obrazek pictureBoxu na null a jeho velikost nastavi na minimalni vzchoyi hodnotu.
        /// </summary>
        /// <param name="pictBox"></param>
        private void ResetPictureBox(PictureBox pictBox)
        {
            pictBox.Image = null;
            pictBox.Size = DEFAULT_PICTUREBOX_SIZE;
        }
        /// <summary>
        /// Inicializace Target Selection a registrace prislusnych udalosti
        /// </summary>
        /// <param name="pictureBox"></param>
        private void SetTargetSelectionEvents(PictureBox pictureBox)
        {
            if (this._selection != null) {
                this._selection.OnTargetSelected -= new TargetSelection.TargetSelectionHandler(selection_OnTargetSelected);
            }
            _selection = new TargetSelection(pictureBox.Image as Bitmap);
            pictureBox.MouseDown += (sender, e) => this._selection.OnMouseDown(sender, e);
            pictureBox.MouseDown += (sender, e) => this._selection.OnMouseDown(sender, e);
            pictureBox.MouseUp += (sender, e) => this._selection.OnMouseUp(sender, e);
            this._selection.OnTargetSelected += new TargetSelection.TargetSelectionHandler(selection_OnTargetSelected);
        }
        private TargetArea _area;


        /// <summary>
        /// Spustí nebo pozastaví sledování objeku
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void btnStart_Click(object sender, EventArgs e)
        {
            if (!(bool)this.btnStart.Tag) {
                this.Start();
            } else {
                this.Stop();
            }
        }
        /// <summary>
        /// Spustení sledování objektu, 
        /// spusti casovac, který zprostředkuje odesilani jednotlivych obrazků sekvence k analyze
        /// </summary>
        private void Start()
        {
            if (this.IsEndSequence || ReferenceEquals(null, this._meanShift))
                return;
            this.btnStart.Tag = true;
            this.btnStart.Text = STOP;
            this.IsPlaying = true;
            this._timerImageSender.Start();
            if (this.IsRealTimeSelected && !this._isRealTrackingRunning && this._meanShift != null) {
                this._isRealTrackingRunning = true;
                this._imageCounter = 0;
            }
            //pouzit kdyz je vyzadavona ukladani snimku v prubehu sledovani
            if (SAVE_IMAGES) {
                ImageManager.NewFolderRequest();
            }
        }
        /// <summary>
        /// Pozastaveni vyhledavani,..zastavuje casovac, ktery odesila obrazky k analyze
        /// </summary>
        private void Stop()
        {
            this.btnStart.Tag = false;
            this.btnStart.Text = START;
            this.IsPlaying = false;
            this._timerImageSender.Stop();
            if (this.IsRealTimeSelected && this._isRealTrackingRunning) {
                this._meanShift = null;
                this._isRealTrackingRunning = false;
                BufferManager.Instance.Reset();
            }
        }
        /// <summary>
        /// Znovunačtení sekvence snímku a posunutí na začátek
        /// </summary>
        private void ResetSequence()
        {
            if (String.IsNullOrEmpty(this._path) || this._selection == null)
                return;
            this._imageCounter = 0;
            this.Stop();
            TrackingDetailsForm.Instance.Clear();
            this._meanShift = null;
            this._endSequence = false;
            this.LoadSingleImage();
            if (this._images != null)
                this._images = null;
            //ImageManager.NewFolderRequest();
            if (this.tcSource.SelectedTab == this.tpRealTimeOT) {
                BufferManager.Instance.Reset();
            }
        }
        private void ResetFrameCapturer()
        {
            if (this._frameCapturer != null) {
                this._frameCapturer.Stop();
                this._frameCapturer.FrameCaptured -= new FrameCapturer.FrameCaptureEventHandler(m_frameCapturer_FrameCaptured);
            }
        }

        /// <summary>
        /// Vygenerovani hlavicky pro soubor diagnostiky
        /// </summary>
        /// <returns></returns>
        private string GetDiagnosticName()
        {
            StringBuilder builder = new StringBuilder();
            builder.AppendLine("====================================================================");
            builder.AppendLine(String.Format("\nTracker Diagnostic ---- Date {0}  Time {1}", DateTime.Now.Date, DateTime.Now.TimeOfDay));
            builder.AppendLine("\nSequence name: " + this._sequenceName);
            builder.AppendLine("\nTracker type: " + this.menuComboBoxTrackers.SelectedItem.ToString().ToUpper());
            builder.AppendLine("\nHistogram Type: " + this.cbHistogramType.SelectedItem.ToString().ToUpper());
            builder.AppendLine(String.Format("\nHistogram Size: [Red] {0} [Green] {1} [Blue] {2} ", this.txeRedBins.Text, this.txeGreenBins.Text, this.txeBlueBins.Text));
            builder.AppendLine(String.Format("\nAdapt when target is lost function is enabled: {0} ", this.IsFirstAidSelected));
            builder.AppendLine(String.Format("\nCalculate simple target area function is enabled {0} ", this.menuItemSimpleSize.Checked));
            builder.AppendLine("====================================================================");

            return builder.ToString();
        }


        /// <summary>
        /// zkusi rozparsovat retezec s textove zadanymi souradnicemi sledovaci oblasti
        /// </summary>
        /// <param name="text">The text.</param>
        /// <returns></returns>
        private Point GetPoint(string text)
        {
            int x, y = 0;
            string[] position = text.Split(',');
            int.TryParse(position[0], out x);
            int.TryParse(position[1], out y);
            return new Point(x, y);
        }

        /// <summary>
        /// Otevre dialog pro vyber cesty pro ukladani diagnostiky
        /// </summary>
        /// <returns> cesta k adresari, kde bude soubor umisten</returns>
        private string OpenDiagnosticDestinationPathDialog()
        {
            FolderBrowserDialog fd = new FolderBrowserDialog();
            fd.ShowNewFolderButton = true;
            fd.Description = "Select destination folder for diagnostic files";
            fd.ShowDialog(this);
            return fd.SelectedPath;
        }
        #region Overriden methos
        protected override void OnClosing(CancelEventArgs e)
        {
            this.ResetFrameCapturer();
            base.OnClosing(e);
            BufferManager.Instance.Reset();
            BackProjectionForm.Instance.ForcedClose();
            TrackingDetailsForm.Instance.ForcedClose();
        }
        protected override void OnClosed(EventArgs e)
        {
            base.OnClosed(e);
            if (this._frameCapturer != null) {
                this._frameCapturer.Dispose();
            }
        }
        #endregion



        #endregion // end methods
    }
}
