﻿using System;
using System.Diagnostics;
using System.Drawing;
using Diagnostic;
using RGBHistogramGenerator;
using RGBHistogramGenerator.Enumerations;
using Tracker.Enumerations;
using TrackerGraphic;

namespace Tracker
{
    /// <summary>
    /// Třída MeanShift,slouží k sledování objektu na jednotlivych obrázcich. Umožnuje vypočitat polohu sledovaneho objektu
    /// a tuto polohu označovat a zaznamenavat.
    /// </summary>
    public class MeanShift
    {
        /// <summary>
        /// Vychozi nastavení poměru změn velikosti okna
        /// </summary>
        private const int DEFAULT_SCALE_RATIO = 5;
        #region Fields
        /// <summary>
        /// Maximální počet iterací
        /// </summary>
        private int _maximumIterations;
        /// <summary>
        /// Delegát k události TargetFound
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public delegate void TrackerHandler(object sender, TrackerEventArgs e);
        /// <summary>
        /// Událost, která odesílá informace o nalezeném cíli
        /// </summary>
        public event TrackerHandler TargetFound;
        /// <summary>
        /// Generátor histogramů
        /// </summary>
        private HistogramGenerator _histGenerator;
        /// <summary>
        /// Target area z předchozí iterace
        /// </summary>
        private TargetArea _perviousArea;
        /// <summary>
        /// Target area současné iterace
        /// </summary>
        private TargetArea _currentArea;
        /// <summary>
        /// Vzorek vyříznutý z předané bitmapy na pozicich Target Area
        /// </summary>
        protected Bitmap _sample;
        /// <summary>
        /// poměr změny velikosti okna
        /// </summary>
        private int m_ratio = DEFAULT_SCALE_RATIO;
        /// <summary>
        /// vzdálenost objektu od objektivu kamery
        /// </summary>
        private double _distance;
        private Stopwatch _stopWatch;
        protected TrackerIterationDiagnostic _iterationDiagnostic;
        private bool _diagnosticMode;
        #endregion //end fields

        #region Properties
        public double DistributionsDistance
        {
            get
            {
                return this._distance;
            }
            protected set
            {
                this._distance = value;
            }
        }
        public Bitmap Sample
        {
            get
            {
                return this._sample;
            }
            protected set
            {
                this._sample = value;
            }
        }
        public TargetArea CurrentArea
        {
            get
            {
                return this._currentArea;
            }
            protected set
            {
                this._currentArea = value;
            }
        }
        public TargetArea PreviousArea
        {
            get
            {
                return this._perviousArea;
            }
            protected set
            {
                this._perviousArea = value;
            }
        }
        public HistogramGenerator HistogramGenerator
        {
            get
            {
                return this._histGenerator;
            }
            protected set
            {
                this._histGenerator = value;
            }
        }
        public int MaximumIterations
        {
            get
            {
                return this._maximumIterations;
            }
            protected set
            {
                this._maximumIterations = value;
            }
        }
        public Stopwatch StopWatch
        {
            get
            {
                return this._stopWatch;
            }
        }
        /// <summary>
        /// Get/Set Poměr změny velikosti okna 
        /// </summary>
        public int ScaleRatio
        {
            get
            {
                return this.m_ratio;
            }
            set
            {
                if (value < 0 || value > 100)
                    throw new FormatException();
                this.m_ratio = value;
            }
        }
        public HistogramType GeneratorType
        {
            get
            {
                return this._histGenerator.GeneratorType;
            }
        }
        public bool EnableDiagnosticMode
        {
            get
            {
                return this._diagnosticMode;
            }
            set
            {
                this._diagnosticMode = value;
            }
        }
        private bool _isSimpleSimpleSizeCalculation = true;
        public bool IsSimpleSizeCalculation
        {
            get
            {
                return this._isSimpleSimpleSizeCalculation;
            }
            set
            {
                this._isSimpleSimpleSizeCalculation = value;
            }
        }
        #endregion // endproperties

        #region Methods
        #region Constructors
        /// <summary>
        /// Konstruktor Mean shift algoritmu, inicializuje generator histogramu, nastavi polohu sledovaneho objektu,
        /// nastavíhorni  mez  poctu iteraci
        /// </summary>
        /// <param name="bCount1">pocet R "binu" pro vygenerovani histogramu</param>
        /// <param name="bCount2">pocet G "binu" pro vygenerovani histogramu</param>
        /// <param name="bCount3">pocet B "binu" pro vygenerovani histogramu</param>
        /// <param name="iterations">maximalni pcoet  mean shift iteraci</param>
        /// <param name="bmp">bitmapa k analyzovani</param>
        /// <param name="area">pozice cíle</param>
        public MeanShift(int numberOfRedBins, int numberOfGreenBins, int numberOfBlueBins, int iterations, HistogramType type, ref Bitmap bmp, TargetArea area)
        {
            _histGenerator = new HistogramGenerator(type);
            this._perviousArea = area;
            if (type != HistogramType.Weighted)
                this._sample = TargetSelection.GetImageSample(ref bmp, area);
            else
                this._sample = TargetSelection.GetImageSample(ref bmp, area.SearchingArea);
            _histGenerator.GenerateHistogram(numberOfRedBins, numberOfGreenBins, numberOfBlueBins, SourceType.Target, ref this._sample, area);
            this._maximumIterations = iterations;
            this._stopWatch = new Stopwatch();
            this._iterationDiagnostic = new TrackerIterationDiagnostic();
            this._iterationDiagnostic.Add(bmp, area, 0);
        }
        #endregion // end constructors
        /// <summary>
        /// Mean - shit Tracker, po nalezení cíle odešle informace o jeho poloze a informace o vypoctu
        /// </summary>
        /// <param name="bmp">Bitmapa k analyze</param>
        public virtual void Tracking(ref Bitmap bmp)
        {
            this._stopWatch.Start();
            int iterations = 0;
            for (int i = 0; i < this._maximumIterations; i++) {
                this._sample = TargetSelection.GetImageSample(ref bmp, this._perviousArea);
                _histGenerator.GenerateHistogram(SourceType.Candidate, ref _sample);
                Point centroid = DistributionCalculator.GetCentroidPosition(_sample, /*this.m_perviousArea,*/
                    this._histGenerator.Target, this._histGenerator.CandidateHistogram);
                if (centroid.X != 0 && centroid.Y != 0)
                    centroid = DistributionCalculator.RecalculateCentroidToArea(centroid, this._perviousArea);
                //data = DistributionCalculator.GetDistributionData(m_sample, CurrentArea, m_histGenerator.Target, m_histGenerator.CandidateHistogram);
                this._currentArea = new AdaptiveTargetArea(this._perviousArea);
                this._currentArea.OffSet(centroid.X, centroid.Y);
                if (TargetArea.CheckBounds(this._currentArea, ref bmp))
                    this._currentArea = this._perviousArea;//new TargetArea(this.m_perviousArea);//predpoklada se, ze je tracker ztracen,
                //nebo cil je mimo meze obrazku a tracker na nej na ztracene pozici "ceka", zamezime posun za hranice obrazku    
                iterations = ++i;
                if ( _currentArea.MatchLocations(_perviousArea) || i == this._maximumIterations) {
                    this.DistanceAdaptation(ref bmp);
                    break;
                }
                this._perviousArea = this._currentArea;

                #region diagnostic
                //if (this.EnableDiagnosticMode)
                //{
                //    Bitmap b = new Bitmap(bmp);
                //    this.m_iterationDiagnostic.Add(b, m_perviousArea, i);
                //}
                #endregion //enddiagnostic

            }
            this._perviousArea = this._currentArea;

            this.OnTargetFound(new TrackerEventArgs(bmp, this._currentArea,
                this._histGenerator.CandidateHistogram, this._histGenerator.GetDistributionDistance(), iterations,
                this._stopWatch.ElapsedMilliseconds));
            this._stopWatch.Reset();
        }
        /// <summary>
        /// Na základe vypocitane vzdalenosti z hodnot cile a kandidata, prizpusobi velikost
        /// prohledavaciho okna tak, aby zmena mela maximalni prospesnost.
        /// </summary>
        /// <param name="bmp"></param>
        protected void DistanceAdaptation(ref Bitmap bmp)
        {
            if (this.m_ratio == 0) //0 indukuje, ze neni pozadavek na zmeny velikosti okna
                return;
            //ziskani vzdalenosti a nasobeno x10, kvuli ziskani rozumne stupnice vzdalenosti
            int calculatedDistance = (int)(this._histGenerator.GetDistributionDistance() * 10);
            AdaptiveTargetArea ta = new AdaptiveTargetArea(this._currentArea);
            float bCoefPrevious = (float)this._histGenerator.MatchHistograms();
            if (this._distance == 0) {
                this._distance = calculatedDistance;
            }
            //indikuje se zda doslo k prechodu na jinou vzdalenost nebo jestli je konstantni + jestli bude zmena prospesna
            if ((calculatedDistance / this._distance) > 1 &&
                    this.UsefulAdaptation(ref bmp, bCoefPrevious, AdaptationType.Reduce)) {
                this._distance = calculatedDistance;
                ta.Reduce(m_ratio);
                this._currentArea = ta;
            }
            if ((calculatedDistance / this._distance) < 1
                /*&& this.UsefulAdaptation(ref bmp, bCoefPrevious,AdaptationType.Expand)*/) {
                this._distance = calculatedDistance;
                ta.Extend(m_ratio);
                this._currentArea = ta;
            }
        }
        /// <summary>
        /// Metoda porovna histogram ze soucasne velikosti okna a noveho okna a vyhodnoti, zda se
        /// vyplati menit rozmery sledovaciho okna. Pokud ano vraci true, jinak flase.
        /// </summary>
        /// <param name="bmp">analyzovana bitmapa</param>
        /// <param name="bCoeft">Bathacharyya koeficient z puvodni velkikosti okna</param>
        /// <param name="type">Typ zmeneni rozmeru redukovat / expandovat</param>
        /// <returns>true v pripade, ze zmena velikosti bude prospesna.</returns>
        private bool UsefulAdaptation(ref Bitmap bmp, float bCoeft, AdaptationType type)
        {
            AdaptiveTargetArea ata = new AdaptiveTargetArea(this._currentArea);
            if (type == AdaptationType.Expand)
                ata.Extend(this.m_ratio);
            else
                ata.Reduce(this.m_ratio);
            Bitmap sample = TargetSelection.GetImageSample(ref bmp, ata);
            this._histGenerator.GenerateHistogram(SourceType.Candidate, ref sample);
            float bc2 = (float)this._histGenerator.MatchHistograms();
            return bc2 >= bCoeft;
        }
        #region vyvolani udalosti
        /// <summary>
        /// Vyvolani udalosti TargetFound
        /// </summary>
        /// <param name="eventArgs">argument udalosti</param>
        protected void OnTargetFound(TrackerEventArgs eventArgs)
        {
            if (null != TargetFound) {
                TargetFound(this, eventArgs);
            }
        }

        #endregion
        #endregion // end methods
    }
}
