﻿using System.Drawing;


namespace TrackerGraphic
{
    /// <summary>
    /// Třída AdaptiveTargetArea je odvozena od třídy TargetArea a je doplněna o funkce změny velikosti
    /// podle předaneho poměru (ratio) zvetseni/zmenseni stran v procentech.
    /// </summary>
    public class AdaptiveTargetArea : TargetArea, IAdaptiveArea
    {
        #region Constants
        /// <summary>
        /// Konstanta predstavujici 100%
        /// </summary>
        private const double HUNDRED_PERCENT = 100.0;
        private readonly Size MINIMUM_SIZE = new Size(3, 3);
#warning 7,7
        public static readonly Size MAXIMUM_SIZE = new Size(300, 200);
        public static readonly Size FIRST_AID = new Size(150, 150);
        private float _orgRatio;
        #endregion //end constants
        public static bool IsFirstAidEnabled;
        #region Constructors
        /// <summary>
        /// Initializes a new instance of the <see cref="AdaptiveTargetArea"/> class.
        /// </summary>
        /// <param name="startPosition">The start position.</param>
        /// <param name="endPosition">The end position.</param>
        public AdaptiveTargetArea(Point startPosition, Point endPosition)
            : base(startPosition, endPosition)
        {
        }
        /// <summary>
        /// Initializes a new instance of the <see cref="AdaptiveTargetArea"/> class.
        /// </summary>
        /// <param name="area">The area.</param>
        public AdaptiveTargetArea(TargetArea area)
            : base(area)
        {
        }
        #endregion // end constructors

        #region Methods
        /// <summary>
        /// Provede zvetseni okna o procentualni hodnotu
        /// </summary>
        /// <param name="ratio">mira zmeny velikosti v procentech</param>
        public void Extend(int ratio)
        {
            Rectangle rec = this.ObjectArea.AreaRectangle;
            Rectangle recS = this.SearchingArea.AreaRectangle;

            Size s = this.GetSize(ratio);
            rec.Inflate(s.Width, s.Height);
            recS.Inflate((int)(s.Width * this.SearchingAreaRatio), (int)(s.Height * this.SearchingAreaRatio));

            this.ObjectArea.AreaRectangle = rec;
            this.SearchingArea.AreaRectangle = recS;
        }
        /// <summary>
        /// Získání nových rozměru současneho sledovaciho okna podle predanych procent velikosti.
        /// Ratio predstavuje miru zvetseni/zmenseni rozmeru sledovaciho okna v procentech.
        /// Metoda vypocita nove rozmery podle teto procentualni miry velikosti.
        /// </summary>
        /// <param name="ratio">mira zmeny velikosti v procentech</param>
        /// <returns>Nova velikost  okna</returns>
        private Size GetSize(int ratio)
        {
            int width = 0;
            int height = 0;
            if (ratio > 0) {
                Rectangle rec = this.ObjectArea.AreaRectangle;
                double width1 = (double)(this.ObjectArea.AreaRectangle.Width / HUNDRED_PERCENT) * ratio;
                double height1 = (double)(this.ObjectArea.AreaRectangle.Height / HUNDRED_PERCENT) * ratio;
                width = (int)width1;
                height = (int)height1;
            }
            return new Size(width, height);
        }
        /// <summary>
        /// Provede zmenseni okna o procentualni hodnotu
        /// </summary>
        /// <param name="ratio">mira zmeny velikosti v procentech</param>
        public void Reduce(int ratio)
        {
            Rectangle rec = this.ObjectArea.AreaRectangle;
            Rectangle recS = this.SearchingArea.AreaRectangle;
            Size s = this.GetSize(ratio);
            rec.Inflate((-1) * s.Width, (-1) * s.Height);
            recS.Inflate((-1) * s.Width * (int)this.SearchingAreaRatio, (-1) * s.Height * (int)this.SearchingAreaRatio);
            this.ObjectArea.AreaRectangle = rec;
            this.SearchingArea.AreaRectangle = recS;
        }
        /// <summary>
        /// Prizpusobi velikost ObjectArea a SearchingArea
        /// </summary>
        /// <param name="size">rozloha oblasti</param>
        public void AdaptObjectArea(Size size)
        {
            if (this.IsUnderMinimum(size)) {
                this.SetFirstAid();
                return;
            }
            if (!this.ObjectArea.AreaRectangle.Size.Equals(size) &&
                !this.IsUnderMinimum(size) &&
                !this.IsOverMaximum(size)) {
                int width = size.Width - this.ObjectArea.AreaRectangle.Width;
                int height = size.Height - this.ObjectArea.AreaRectangle.Height;
                Rectangle rec = this.ObjectArea.AreaRectangle;
                Rectangle recS = this.SearchingArea.AreaRectangle;
                width /= 2;
                height /= 2;
                rec.Inflate(width, height);
                recS.Inflate(width, height);
                this.ObjectArea.AreaRectangle = rec;
                this.SearchingArea.AreaRectangle = recS;
                if (!this.ObjectArea.Center.Equals(this.SearchingArea.Center)) {
                    Recenter();
                }
                if (!this.CheckAreaRatio())
                    this.ResetSearchingArea();
            }
        }
        /// <summary>
        /// Metoda se stará o znovu vycentrování Prohledavaci oblasti (Searching area) na
        /// stred oblasti cile (Object area)
        /// </summary>
        /// <returns>true v pripade uspesneho vycentrovani</returns>
        private bool Recenter()
        {
            Point p = new Point();
            p.X = this.ObjectArea.Center.X - this.SearchingArea.Center.X;
            p.Y = this.ObjectArea.Center.Y - this.SearchingArea.Center.Y;
            this.SearchingArea.OffSet(p);
            return this.SearchingArea.Center.Equals(this.ObjectArea.Center);
        }
        /// <summary>
        /// Zkontroluje zda nastavovana oblast neni pod minimalni nastavenou velikosti okna
        /// </summary>
        /// <param name="size">rozloha oblasti</param>
        /// <returns>true v pripade, ze nove okno je mensi nez minimalni povolene</returns>
        private bool IsUnderMinimum(Size size)
        {
            return (size.Width <= this.MINIMUM_SIZE.Width) ? true :
                (size.Height <= this.MINIMUM_SIZE.Height) ? true : false;
        }
        /// <summary>
        /// Zkontroluje zda nastavovana oblast neni nad maximalni nastavenou velikosti okna
        /// </summary>
        /// <param name="size">rozloha oblasti</param>
        /// <returns>true v pripade, ze nove okno je vetsi nez maximalni povolene</returns>
        private bool IsOverMaximum(Size size)
        {
            return (size.Width >= MAXIMUM_SIZE.Width) ? true :
                (size.Height >= MAXIMUM_SIZE.Height) ? true : false;
        }
        /// <summary>
        /// Nastavi rozlu okna  na pozadovanou velikost - v pripade, ze se predpoklada,
        /// ze je cil ztracen
        /// </summary>
        private void SetFirstAid()
        {
            if (IsFirstAidEnabled) {
                this.AdaptObjectArea(FIRST_AID);
            }
        }
        #endregion // end methods

    }
}
