﻿using System;
using System.Drawing;

namespace TrackerGraphic
{
    public class TargetArea
    {
        /// <summary>
        /// vzdálenost prohledavaciho okna od okna ohranicujici objekt
        /// </summary>
        public const int SEARCHING_AREA_RATIO = 2;     //3


        #region Fields
        /// <summary>
        /// Oblast cíle
        /// </summary>
        private Area _objectArea;
        /// <summary>
        /// Prohledavací oblast
        /// </summary>
        private Area _searchingArea;
        /// <summary>
        /// Pomer zvetseni velikosti prohledavaciho oblasti k oblasti Objektu
        /// </summary>
        private int _searching_area_ratio = SEARCHING_AREA_RATIO;
        #endregion

        #region Properties
        /// <summary>
        /// Get / Set  Object area
        /// </summary>
        public Area ObjectArea
        {
            get
            {
                return this._objectArea;
            }
            set
            {
                this._objectArea = value;
            }
        }
        /// <summary>
        /// Get / Set Searching area
        /// </summary>
        public Area SearchingArea
        {
            get
            {
                return this._searchingArea;
            }
            set
            {
                this._searchingArea = value;
            }
        }
        /// <summary>
        /// Get / Set Stred Cilove oblasti
        /// </summary>
        public Point Center
        {
            get
            {
                return this.ObjectArea.Center;
            }
            protected set
            {
                this._objectArea.Center = value;
            }
        }
        /// <summary>
        /// Get / Set Pomer Searching area k Target area
        /// </summary>
        /// <value>
        /// pomer
        /// </value>
        public int SearchingAreaRatio
        {
            get
            {
                return this._searching_area_ratio;
            }
            set
            {
                if (value < 1)
                    value = SEARCHING_AREA_RATIO;
                this._searching_area_ratio = value;
            }
        }
        #endregion //end properties

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the <see cref="TargetArea"/> class.
        /// </summary>
        /// <param name="start">The start.</param>
        /// <param name="end">The end.</param>
        public TargetArea(Point start, Point end)
        {
            this._objectArea = new Area(start, end);
            this.SearchingAreaRatio = SEARCHING_AREA_RATIO;
            this.GenerateSearchingArea(this.SearchingAreaRatio);
        }
        /// <summary>
        /// Initializes a new instance of the <see cref="TargetArea"/> class.
        /// </summary>
        /// <param name="area">The area.</param>
        public TargetArea(TargetArea area)
        {
            this._objectArea = new Area(area.ObjectArea.AreaRectangle.Location, area._objectArea.BottomRightCorner);
            if (area._objectArea.AreaRectangle.Height != _objectArea.AreaRectangle.Height || area._objectArea.AreaRectangle
                .Width != _objectArea.AreaRectangle.Width)
                throw new ArgumentException();

            this._searchingArea = new Area(area._searchingArea.AreaRectangle);
            this.Center = area.Center;
            this.ObjectArea.OffSetPoint = area.ObjectArea.OffSetPoint;
            this.SearchingArea.OffSetPoint = area.SearchingArea.OffSetPoint;
        }
        #endregion // end constructors

        #region Methods
        /// <summary>
        /// Generates the searching area.
        /// </summary>
        /// <param name="distance">The distance.</param>
        private void GenerateSearchingArea(int ratio)
        {
            int height = (this.ObjectArea.AreaRectangle.Height * ratio);
            int width = (this.ObjectArea.AreaRectangle.Width * ratio);
            Rectangle rec = this.ObjectArea.AreaRectangle;
            rec.Inflate((width - this.ObjectArea.AreaRectangle.Width) / 2,
                (height - this.ObjectArea.AreaRectangle.Height) / 2);
            this._searchingArea = new Area(rec);
        }
        /// <summary>
        /// Posunuti oblasti  o novou pozici
        /// </summary>
        /// <param name="position">The position.</param>
        public void OffSet(Point position)
        {
            this.ObjectArea.OffSet(position);
            this.SearchingArea.OffSet(position);
            if (!this.CheckAreaRatio())
                this.ResetSearchingArea();
        }
        /// <summary>
        /// Posunuti oblasti  o novou pozici x ,y
        /// </summary>
        /// <param name="x">souradnice x</param>
        /// <param name="y">souradnice y</param>
        public void OffSet(int x, int y)
        {
            this.OffSet(new Point(x, y));
        }
        /// <summary>
        /// Returns a <see cref="System.String"/> that represents this instance.
        /// </summary>
        /// <returns>
        /// A <see cref="System.String"/> that represents this instance.
        /// </returns>
        public override string ToString()
        {
            return String.Format("Height {0}  Width {1} Upper left {2}  Bottom right {3}",
                this._objectArea.AreaRectangle.Height, this._objectArea.AreaRectangle.Width,
                this._objectArea.AreaRectangle.Location, this._objectArea.BottomRightCorner);
        }

        /// <summary>
        /// Zjisti zda je ObjectArea na hranici obrazku
        /// </summary>
        /// <param name="area">oblast cile</param>
        /// <param name="bmp">obrazek</param>
        /// <returns>true v pripade ze je Oblast na hranici obrazku</returns>
        public static bool CheckBounds(TargetArea area, ref Bitmap bmp)
        {
            return CheckBounds(area.ObjectArea, ref bmp);
        }
        /// <summary>
        /// Zjisti zda je predana oblast na hranici obrazku
        /// </summary>
        /// <param name="area">oblast</param>
        /// <param name="bmp">obrazek</param>
        /// <returns>true v pripade ze je Oblast na hranici obrazku</returns>
        public static bool CheckBounds(Area area, ref Bitmap bmp)
        {
            return CheckBounds(area, bmp.Size);
        }

        public static bool CheckBounds(Area area, Size size)
        {
            bool result;
            return result = (area.UpperLeftCorner.X <= 0) ? true :
                 (area.UpperLeftCorner.Y <= 0) ? true :
                 (area.BottomRightCorner.X >= size.Width) ? true :
                 (area.BottomRightCorner.Y >= size.Height) ? true : false;
        }
        /// <summary>
        /// Srovna jestli se predesla a soucasna poloha teziste shoduji,
        /// ... v tomto pripade mean shift konci, nasel spravne teziste
        /// </summary>
        /// <returns>true v pripade absolutni shody obou pozici</returns>
        public bool MatchLocations(TargetArea area)
        {
            return this.Center.Equals(area.Center);
        }
        /// <summary>
        /// Zkontroluje pomer velikosti oken searching area a object area
        /// </summary>
        /// <returns>true v pripade, ze jsou nastavene korektni velikosti</returns>
        public bool CheckAreaRatio()
        {
            return (this.SearchingArea.Width ==
                (this._searching_area_ratio * this.ObjectArea.Width)) ?
                (this.SearchingArea.Height ==
                (this._searching_area_ratio * this.ObjectArea.Height)) : false;
        }
        /// <summary>
        /// Prenastavi rozmery Searching area k pomeru velikosti
        /// </summary>
        protected void ResetSearchingArea()
        {
            int valid = (int)(this._objectArea.Width * this._searching_area_ratio);
            if (!this.SearchingArea.Width.Equals(valid) ||
                !this.SearchingArea.Height.Equals(valid)) {
                int differenceX = valid - this.SearchingArea.AreaRectangle.Width;
                int differenceY = valid - this.SearchingArea.AreaRectangle.Height;
                differenceX /= 2;
                differenceY /= 2;
                if (differenceX != 0 || differenceY != 0)
                    this.SearchingArea.Inflate(differenceX, differenceY);
            }
        }
        #endregion //end methods

    }
}
