﻿using System.Drawing;
using System.Windows.Forms;

namespace TrackerGraphic
{
    /// <summary>
    /// Třída sloužící k získání informací o pozici vybraného sledovaného objektu
    /// </summary>
    public class TargetSelection
    {
        #region Fields
        /// <summary>
        /// vybraný cíl
        /// </summary>
        private Bitmap _selectedTarget;
        /// <summary>
        /// Původní obrazek z kterého byl cíl získan
        /// </summary>
        private Bitmap _orginalImage;
        /// <summary>
        /// Pozice na niz se zacal oznacovat cil
        /// </summary>
        private Point _startPosition;
        /// <summary>
        /// Oblast ohranicujici cíl
        /// </summary>
        private Rectangle _rectangle;
        #endregion //end fields

        #region Delegates + Events
        /// <summary>
        /// Delegat k udalosti OnTargetSelected
        /// </summary>
        public delegate void TargetSelectionHandler(TargetSelection sender, TargetSelectionEventArgs e);
        /// <summary>
        /// Událost, jenž je odpálena v případě výběru sledovaného objektu
        /// </summary>
        public event TargetSelectionHandler OnTargetSelected;
        #endregion //delegates + events

        #region Properties
        /// <summary>
        /// Get Počáteční pozice výběru
        /// </summary>
        public Point StartPosition
        {
            get
            {
                return this._startPosition;
            }
        }
        #endregion //endproperties

        #region Constructors
        /// <summary>
        /// Initializes a new instance of the <see cref="TargetSelection"/> class.
        /// </summary>
        /// <param name="original">The original.</param>
        public TargetSelection(Bitmap original)
        {
            this._orginalImage = original;
        }
        #endregion //end constructors

        #region Methods
        /// <summary>
        /// Obslužná metoda události OnMouseUp, ziska konecne souradnice polohy objektu a 
        /// na zaklade souradnic vytahne obrazek cíle
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void OnMouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button != MouseButtons.Left)
                return;
            Size s = new Size(e.X - _startPosition.X, e.Y - _startPosition.Y);
            if ((s.Height > 0 && s.Width > 0)) {
                this._rectangle = new Rectangle(_startPosition, s);
                this.OnSelected(this._selectedTarget, this._rectangle);
            }
        }
        /// <summary>
        /// Obslužná metoda události OnMouseDown - zaznamena pri stisku leveho tlacitka pocatecni pozici
        /// cile
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        public void OnMouseDown(object sender, MouseEventArgs e)
        {
            if (e.Button == MouseButtons.Left)
                this._startPosition = e.Location;
        }
        /// <summary>
        /// Odpaleni udalosti Target selected
        /// </summary>
        /// <param name="bmp">bitmapa s cilem</param>
        /// <param name="rec">rectangle</param>
        private void OnSelected(Bitmap bmp, Rectangle rec)
        {
            if (null != OnTargetSelected) {
                this.OnTargetSelected(this, new TargetSelectionEventArgs(bmp, rec));
            }
        }
        /// <summary>
        /// Z předané bitmapy udělá výsek sledovaného objektu podle předaných souřadnic.
        /// </summary>
        /// <param name="bmp">zdrojova bitmapa</param>
        /// <param name="area">souřadnice cíle</param>
        /// <returns>výsek daného obrázku</returns>
        public static Bitmap GetImageSample(ref Bitmap bmp, TargetArea area)
        {
            CheckBounds(ref bmp, area);
            return bmp.Clone(area.ObjectArea.AreaRectangle, bmp.PixelFormat);
        }
        public static Bitmap GetImageSample(ref Bitmap bmp, TargetArea area, ref Bitmap wholeImage)
        {
            CheckBounds(ref wholeImage, area);
            Rectangle rec;
            if (!bmp.Width.Equals(wholeImage.Width) && !bmp.Height.Equals(wholeImage.Height)) {
                rec = RecalculateToSearchingArea(area, area.ObjectArea.Center);
            } else
                rec = area.ObjectArea.AreaRectangle;
            return bmp.Clone(rec, bmp.PixelFormat);
        }
        /// <summary>
        /// Vrati vysek z predane bitmapy, podle sledovaci oblasti cile.
        /// </summary>
        /// <param name="bmp">cast obrazu ke zpracovani</param>
        /// <param name="searchLocation">sledovaci oblast</param>
        /// <param name="area">sledovaci oblast cile</param>
        /// <param name="wholeImage">cely obraz</param>
        /// <returns></returns>
        public static Bitmap GetImageSample(ref Bitmap bmp, Point searchLocation, TargetArea area, ref Bitmap wholeImage)
        {
            CheckBounds(ref wholeImage, area);
            Rectangle rec;
            if (!bmp.Width.Equals(wholeImage.Width) && !bmp.Height.Equals(wholeImage.Height)) {
                searchLocation.X = area.ObjectArea.UpperLeftCorner.X - searchLocation.X;
                searchLocation.Y = area.ObjectArea.UpperLeftCorner.Y - searchLocation.Y;
                rec = new Rectangle(searchLocation.X, searchLocation.Y,
                    area.ObjectArea.AreaRectangle.Width, area.ObjectArea.AreaRectangle.Height);
            } else
                rec = area.ObjectArea.AreaRectangle;
            return bmp.Clone(rec, bmp.PixelFormat);
        }
        private static Rectangle RecalculateToSearchingArea(TargetArea area, Point center)
        {
            Point nCenter = new Point(center.X, center.Y);
            nCenter.X = (int)(center.X / area.ObjectArea.AreaRectangle.Width);
            nCenter.Y = (int)(center.Y / area.ObjectArea.AreaRectangle.Height);
            int ratio = (int)area.SearchingAreaRatio;
            Point newPoint = new Point((center.X + ratio / 2),
                (center.Y + ratio / 2));
            Point recLocation = new Point(newPoint.X - (area.ObjectArea.AreaRectangle.Width / 2),
                newPoint.Y - (area.ObjectArea.AreaRectangle.Height / 2));
            Rectangle newRec = new Rectangle(recLocation, area.ObjectArea.AreaRectangle.Size);
            return newRec;
        }
        /// <summary>
        /// Z předané bitmapy udělá výsek sledovaného objektu podle předaných souřadnic.
        /// S tim rozdilem, ze oblast nijak neposouva, ani nemeni. V pripade, ze je oblast
        /// za hranici obrazku, jednoduse vezme  jen tu cast, ktera je v mezich obrazku
        /// </summary>
        /// <param name="bmp">cely obrazek</param>
        /// <param name="area">oblast podle ktere ma byt udelan vzorek</param>
        /// <returns>vyseknuty vzorek na miru velikosti oblasti</returns>
        public static Bitmap GetImageSample(ref Bitmap bmp, Area area)
        {
            return bmp.Clone(GetValidRectangle(area, bmp.Size), bmp.PixelFormat);
        }
        /// <summary>
        /// Kontrola zda se oblast cile neposunula za hranice obrazku,
        /// v takovem pripade posune oblast cile tak, aby nepresahla hranici obrazku
        /// </summary>
        /// <param name="bmp">bitmapa</param>
        /// <param name="area">oblast zajmu</param>
        /// <returns></returns>
        private static void CheckBounds(ref Bitmap bmp, TargetArea area)
        {
            CheckBounds(ref bmp, area.ObjectArea);
        }
        private static void CheckBounds(ref Bitmap bmp, Area area)
        {
            //provedeny zmeny: zadny navratovy typ, predtim vracel TargetArea
            int difference;
            if (area.BottomRightCorner.X > bmp.Width) {
                difference = bmp.Width - area.BottomRightCorner.X;
                area.OffSet(difference, 0);
            }
            if (area.BottomRightCorner.Y > bmp.Height) {
                difference = bmp.Height - area.BottomRightCorner.Y;
                area.OffSet(0, difference);
            }
            if (area.UpperLeftCorner.X < 0) {
                difference = (-1) * area.UpperLeftCorner.X;
                area.OffSet(difference, 0);
            }
            if (area.UpperLeftCorner.Y < 0) {
                difference = (-1) * area.UpperLeftCorner.Y;
                area.OffSet(0, difference);

            }
        }

        /// <summary>
        /// Vrátí validní oblast, v pripade, ze predana oblast, presahuje predane hranice.
        /// Oblast se zmensi tak, ze zustane v predanych mezich 
        /// </summary>
        /// <param name="area">oblast k validaci</param>
        /// <param name="size">rozloha ke ktere zjistujeme validaci</param>
        /// <returns>validni oblast v mezich obrazku</returns>
        public static Rectangle GetValidRectangle(Area area, Size size)
        {
            if (!TargetArea.CheckBounds(area, size))
                return area.AreaRectangle;
            Rectangle rec = area.AreaRectangle;
            Point location = area.UpperLeftCorner;
            if (area.UpperLeftCorner.X <= 0) {
                rec.Width = area.UpperLeftCorner.X + area.AreaRectangle.Width;
                location.X = 0;
            }
            if (area.BottomRightCorner.X >= size.Width) {
                rec.Width = area.AreaRectangle.Width - (area.BottomRightCorner.X - size.Width);
                int difference = size.Width - rec.Width;
                location.X = difference < 0 ? 0 : difference;
            }
            if (area.UpperLeftCorner.Y <= 0) {
                rec.Height = area.UpperLeftCorner.Y + area.AreaRectangle.Height;
                location.Y = 0;
            }
            if (area.BottomRightCorner.Y >= size.Height) {
                rec.Height = area.AreaRectangle.Height - (area.BottomRightCorner.Y - size.Height);
                location.Y = area.AreaRectangle.Location.Y < 0 ? 0 : area.AreaRectangle.Location.Y;

            }
            rec.Location = location;
            return ValidateSize(rec, size);
        }
        private static Rectangle ValidateSize(Rectangle rectangle, Size size)
        {
            rectangle.Width = rectangle.Width > size.Width ? size.Width : rectangle.Width;
            rectangle.Height = rectangle.Height > size.Height ? size.Height : rectangle.Height;
            return rectangle;
        }

        #endregion // endmethods



    }
}
