﻿using System;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using AForge.Video.DirectShow;
using FrameGrabber.Properties;
using UI;

namespace FrameGrabber
{
    public partial class FormFrameGrabber : Form
    {
        private const string EXTENSION = ".jpg";
        private const string START = "Start";
        private const string STOP = "Stop";
        private const string RESOLUTION_LOW = "320x240";
        private const string RESOLUTION_MIDDLE = "640x480";
        private const string RESOLUTION_HIGH = "1024x768";


        private FrameCapturer _grabber;
        private DirectoryInfo _directoryDestination;
        private ErrorProvider _errorProvider;
        private bool _isLocked = true;
        private int _frameCounter;
        private DeviceDetectedForm _devicesForm;
        private Size _selectedSize = new Size(320, 240);
        private string _progressText;
        private System.Windows.Forms.Timer _progressTimer;



        public string ProgressText
        {
            get
            {
                return _progressText;
            }
            set
            {
                _progressText = value;
            }
        }

        public FormFrameGrabber()
        {
            InitializeComponent();
            progressBar1.Visible = false;
            _progressTimer = new System.Windows.Forms.Timer();
            progressBar1.Maximum = 100;
            progressBar1.Minimum = 0;
            _progressTimer.Interval = 200;
            _progressTimer.Tick += new EventHandler(_progressTimer_Tick);
            _grabber = new FrameCapturer();
            _errorProvider = new ErrorProvider();
            cbResolution.DataSource = new string[] { RESOLUTION_LOW, RESOLUTION_MIDDLE/*, RESOLUTION_HIGH*/ };
            cbResolution.SelectedIndex = 0;
            this.Shown += new EventHandler(FormFrameGrabber_Shown);
            this.cbResolution.SelectedValueChanged += new EventHandler(cbResolution_SelectedValueChanged);
        }

        void _progressTimer_Tick(object sender, EventArgs e)
        {
            if (progressBar1.Value == progressBar1.Maximum) {
                progressBar1.Value = 0;
            }
            progressBar1.Value += 10;
        }




        void cbResolution_SelectedValueChanged(object sender, EventArgs e)
        {
            _selectedSize = this.ParseStringResolution(this.cbResolution.SelectedValue.ToString());
        }

        void FormFrameGrabber_Shown(object sender, EventArgs e)
        {
            InitializeGrabber();
        }

        void _grabber_DeviceFound(object sender, EventArgs e)
        {
            DevicesEventArgs args = e as DevicesEventArgs;
            if (args.Devices.Count == 1) {
                _grabber.SetDevice(0);
            } else {
                ShowDevicesForm(args.Devices);
            }
            ActivateCamera();
            _grabber.DeviceFound -= new EventHandler(_grabber_DeviceFound);
        }

        void _devicesForm_DeviceSelected(object sender, EventArgs e)
        {
            _devicesForm.DeviceSelected -= new EventHandler(_devicesForm_DeviceSelected);
            _grabber.SetDevice((e as SelectedDeviceEventArgs).SelectedIndex);
            ActivateCamera();
        }
        DirectoryInfo _dirInfo;
        /// <summary>
        /// Handles the Click event of the btnOpenFolder control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btnOpenFolder_Click(object sender, EventArgs e)
        {
            _isLocked = true;

            _errorProvider.SetError(this.btnOpenFolder, string.Empty);
            FolderBrowserDialog dlg = new FolderBrowserDialog();
            if (!String.IsNullOrEmpty(Settings.Default.DefaultPath)) {
                dlg.SelectedPath = Settings.Default.DefaultPath;
            }
            dlg.ShowNewFolderButton = true;
            dlg.ShowDialog(this);
            _dirInfo = new DirectoryInfo(dlg.SelectedPath);
            if (_dirInfo.Exists) {
                _directoryDestination = _dirInfo;
                Settings.Default.DefaultPath = _dirInfo.FullName;
            } else {
                _errorProvider.BlinkRate = 500;
                _errorProvider.SetError(this.btnOpenFolder, "Selected path does not exist.");
            }
        }

        /// <summary>
        /// Handles the Click event of the btnStart control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void btnStart_Click(object sender, EventArgs e)
        {
            if (btnStart.Text == START) {
                if (!_grabber.IsDeviceSet) {
                    _grabber.Reset();
                    this._grabber.DeviceFound += new EventHandler(_grabber_DeviceFound);
                    return;
                }
                _errorProvider.SetError(sender as Control, string.Empty);
                if (_dirInfo == null || !_dirInfo.Exists) {
                    _errorProvider.SetError(sender as Control, "Destination folder path is not set.");
                    return;
                }
                if (cbResolution.SelectedItem != null && cbResolution.SelectedItem is String) {
                    this._grabber.FrameSize = this.ParseStringResolution(this.cbResolution.SelectedItem.ToString());
                }
                _isLocked = false;
                btnStart.Text = STOP;
                this._progressTimer.Start();
                progressBar1.Visible = true;

            } else {
                btnStart.Text = START;
                Stop();
                this._progressTimer.Stop();
                progressBar1.Visible = false;
                progressBar1.Value = 0;

            }


        }

        /// <summary>
        /// Initializes the grabber.
        /// </summary>
        private void InitializeGrabber()
        {
            if (_grabber == null) {
                _grabber = new FrameCapturer();
            }

            _grabber.DeviceFound += new EventHandler(_grabber_DeviceFound);
            _grabber.FrameCaptured += new FrameCapturer.FrameCaptureEventHandler(_grabber_FrameCaptured);
        }
        /// <summary>
        /// Handles the FrameCaptured event of the _grabber control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="AForge.Video.NewFrameEventArgs"/> instance containing the event data.</param>
        void _grabber_FrameCaptured(object sender, AForge.Video.NewFrameEventArgs e)
        {
            Bitmap bmp = e.Frame.Clone() as Bitmap;
            bmp = bmp.GetThumbnailImage(_selectedSize.Width, _selectedSize.Height, null, IntPtr.Zero) as Bitmap;
            this.pictureBox1.BeginInvoke(new Action(() => {
                if (!pictureBox1.Size.Equals(bmp.Size)) {
                    int hDiff = bmp.Size.Height - pictureBox1.Size.Height;
                    int wDiff = bmp.Width - pictureBox1.Size.Width;
                    hDiff = this.Size.Height + hDiff;
                    wDiff = this.Size.Width + wDiff;
                    this.Size = new Size(wDiff, hDiff);
                    pictureBox1.Size = new Size(bmp.Size.Width, bmp.Size.Height);
                    progressBar1.Location = new Point(pictureBox1.Location.X, pictureBox1.Bottom + 10);
                    progressBar1.Width = pictureBox1.Size.Width;
                }
                this.pictureBox1.Image = bmp;
            }));
            if (!_isLocked) {
                SaveImage(e.Frame.GetThumbnailImage(_selectedSize.Width, _selectedSize.Height, null, IntPtr.Zero) as Bitmap);
            }
            e.Frame.Dispose();

        }
        /// <summary>
        /// Saves the image.
        /// </summary>
        /// <param name="bmp">The BMP.</param>
        private void SaveImage(object argument)
        {
            Bitmap bmp = argument as Bitmap;
            string fileName = _directoryDestination + @"\" + (_frameCounter++).ToString() + EXTENSION;
            bmp.Save(fileName);
            bmp.Dispose();
        }

        /// <summary>
        /// Stops this instance.
        /// </summary>
        private void Stop()
        {
            _isLocked = true;
            _frameCounter = 0;
            _dirInfo = null;
        }
        /// <summary>
        /// Shows the devices form.
        /// </summary>
        /// <param name="devicesList">The devices list.</param>
        private void ShowDevicesForm(FilterInfoCollection devicesList)
        {
            _devicesForm = new DeviceDetectedForm(devicesList);
            _devicesForm.DeviceSelected += new EventHandler(_devicesForm_DeviceSelected);
            _devicesForm.StartPosition = FormStartPosition.CenterParent;
            _devicesForm.ShowDialog(this);
        }
        /// <summary>
        /// Activates the camera.
        /// </summary>
        private void ActivateCamera()
        {
            if (_grabber.IsDeviceSet) {
                _grabber.Start();
            }
        }

        protected override void OnClosed(EventArgs e)
        {
            this._grabber.Stop();
            this._grabber.Dispose();
            Settings.Default.Save();
            base.OnClosed(e);
        }

        private Size ParseStringResolution(string res)
        {
            Size s = new Size();
            s.Width = 320;
            s.Height = 240;
            string[] parts = res.Split('x');
            int width = 0;
            int height = 0;
            if (parts.Length == 2) {
                if (!String.IsNullOrEmpty(parts[0]) && int.TryParse(parts[0], out width)) {
                    s.Width = width;
                }
                if (!String.IsNullOrEmpty(parts[1]) && int.TryParse(parts[1], out height)) {
                    s.Height = height;
                }
            }
            return s;
        }


    }

    public static class SafeHelper
    {


        public static void SetText(this Label varLabel, string newText)
        {
            if (varLabel.InvokeRequired) {
                varLabel.BeginInvoke(new MethodInvoker(() => SetText(varLabel, newText)));
            } else {
                varLabel.Text = newText;
            }
        }
    }
}
