﻿using System;
using System.Drawing;
using System.Windows.Forms;
using AForge.Video;
using AForge.Video.DirectShow;

namespace UI
{
    public sealed class FrameCapturer : IDisposable
    {

        private static FrameCapturer m_frameCapturer;
        private VideoCaptureDevice m_vcDevice;
        private FilterInfoCollection m_filterCollection;
        private Timer m_camDetector;
        private int m_devicesCount;

        public delegate void FrameCaptureEventHandler(object sender, NewFrameEventArgs e);
        public event FrameCaptureEventHandler FrameCaptured;
        public event EventHandler DeviceFound;
        /// <summary>
        /// Gets the video capture devices.
        /// </summary>
        /// <value>The video capture devices.</value>
        public VideoCaptureDevice VideoCaptureDevices
        {
            get
            {
                return this.m_vcDevice;
            }
        }
        /// <summary>
        /// Gets the filter info collection.
        /// </summary>
        /// <value>The filter info collection.</value>
        public FilterInfoCollection FilterInfoCollection
        {
            get
            {
                return this.m_filterCollection;
            }
        }
        /// <summary>
        /// Gets a value indicating whether this instance is running.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is running; otherwise, <c>false</c>.
        /// </value>
        public bool IsRunning
        {
            get
            {
                return this.m_vcDevice.IsRunning;
            }
        }
        /// <summary>
        /// Gets a value indicating whether is device set.
        /// </summary>
        /// <value>
        /// 	<c>true</c> if this instance is device set; otherwise, <c>false</c>.
        /// </value>
        public bool IsDeviceSet
        {
            get
            {
                return !String.IsNullOrEmpty(this.m_vcDevice.Source);
            }
        }
        private Size _frameSize;

        public Size FrameSize
        {
            get
            {
                return _frameSize;
            }
            set
            {
                _frameSize = value;
                m_vcDevice.DesiredFrameSize = value;
                Reset();
            }
        }
        private static bool _isDefaultResoulutionSelected;

        public static bool IsDefaultResoulutionSelected
        {
            get
            {
                return _isDefaultResoulutionSelected;
            }
            set
            {
                _isDefaultResoulutionSelected = value;
            }
        }

        public int FrameRate
        {
            get
            {
                if (m_vcDevice != null) {
                    return this.m_vcDevice.DesiredFrameRate;
                } else {
                    return 0;
                }
            }
            set
            {
                if (m_vcDevice != null) {
                    this.m_vcDevice.DesiredFrameRate = value;
                }
            }


        }


        /// <summary>
        /// Initializes a new instance of the <see cref="FrameCapturer"/> class.
        /// </summary>
        public FrameCapturer()
        {
            this.m_vcDevice = new VideoCaptureDevice();
            m_vcDevice.DesiredFrameRate = 50;
            this.m_filterCollection = new FilterInfoCollection(FilterCategory.VideoInputDevice);
            this.m_vcDevice.NewFrame += new NewFrameEventHandler(m_vcDevice_NewFrame);
            this.m_camDetector = new Timer();
            this.m_camDetector.Interval = 1000;
            this.m_camDetector.Tick += new EventHandler(m_camDetector_Tick);
            this.Reset();
        }

        public void Reset()
        {
            //if (FrameCapturer.IsDefaultResoulutionSelected) {
            //    this.m_vcDevice.DesiredFrameSize = new Size(320, 240);
            //} else {
            //    this.m_vcDevice.DesiredFrameSize = new Size(640, 480);
            //}
            this.m_devicesCount = 0;
            this.m_camDetector.Start();
        }
        /// <summary>
        /// Handles the Tick event of the m_camDetector control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void m_camDetector_Tick(object sender, EventArgs e)
        {
            this.m_filterCollection = new FilterInfoCollection(FilterCategory.VideoInputDevice);
            if (this.m_filterCollection.Count != this.m_devicesCount) {
                this.m_devicesCount = this.m_filterCollection.Count;
                this.OnDeviceFound(new DevicesEventArgs(this.m_filterCollection));
            } else {
                if (this.m_filterCollection.Count == 0 && this.IsRunning)
                    this.Stop();
            }
        }
        /// <summary>
        /// Handles the NewFrame event of the m_vcDevice control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="eventArgs">The <see cref="AForge.Video.NewFrameEventArgs"/> instance containing the event data.</param>
        void m_vcDevice_NewFrame(object sender, NewFrameEventArgs eventArgs)
        {
            this.OnFrameCaptured(eventArgs);
        }
        /// <summary>
        /// Gets the instance.
        /// </summary>
        /// <returns></returns>
        public static FrameCapturer GetInstance()
        {
            if (m_frameCapturer == null)
                m_frameCapturer = new FrameCapturer();
            m_frameCapturer.Reset();
            return m_frameCapturer;
        }
        /// <summary>
        /// Sets the device.
        /// </summary>
        /// <param name="index">The index.</param>
        public void SetDevice(int index)
        {
            if (index <= m_filterCollection.Count - 1) {
                this.m_vcDevice.Source = m_filterCollection[index].MonikerString;
            }
        }
        /// <summary>
        /// Raises the <see cref="E:FrameCaptured"/> event.
        /// </summary>
        /// <param name="e">The <see cref="AForge.Video.NewFrameEventArgs"/> instance containing the event data.</param>
        private void OnFrameCaptured(NewFrameEventArgs e)
        {
            if (this.FrameCaptured != null) {
                this.FrameCaptured(this, e);
            }
        }
        /// <summary>
        /// Raises the <see cref="E:DeviceFound"/> event.
        /// </summary>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void OnDeviceFound(EventArgs e)
        {
            if (DeviceFound != null)
                DeviceFound(this, e);
        }
        /// <summary>
        /// Starts this instance.
        /// </summary>
        public void Start()
        {
            if (!this.m_vcDevice.IsRunning && this.m_vcDevice.Source != null)
                this.m_vcDevice.Start();
        }
        /// <summary>
        /// Stops this instance.
        /// </summary>
        public void Stop()
        {
            if (this.m_vcDevice.IsRunning) {
                this.m_camDetector.Stop();
                this.m_camDetector.Tick -= new EventHandler(m_camDetector_Tick);
                this.m_vcDevice.NewFrame -= new NewFrameEventHandler(m_vcDevice_NewFrame);
                System.Threading.ThreadPool.QueueUserWorkItem(s => {
                    try {
                        this.m_vcDevice.Stop();
                    } catch (Exception ex) {
                        System.Diagnostics.Debug.WriteLine(ex.Message);
                    }

                });
            }
        }

        public void ChangeResolution(bool defaultMode)
        {
            FrameCapturer.IsDefaultResoulutionSelected = defaultMode;

        }

        public void ChangeFrameRate(int rate)
        {
            this.m_vcDevice.DesiredFrameRate = rate;
        }
        #region IDisposable Members

        /// <summary>
        /// Performs application-defined tasks associated with freeing, releasing, or resetting unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            if (this.m_vcDevice.IsRunning) {
            }
        }

        #endregion
    }
}
